/* Secret Labs' Regular Expression Engine
 *
 * regular expression matching engine
 *
 * partial history:
 * 1999-10-24 fl   created (based on existing template matcher code)
 * 2000-03-06 fl   first alpha, sort of
 * 2000-08-01 fl   fixes for 1.6b1
 * 2000-08-07 fl   use PyOS_CheckStack() if available
 * 2000-09-20 fl   added expand method
 * 2001-03-20 fl   lots of fixes for 2.1b2
 * 2001-04-15 fl   export copyright as Python attribute, not global
 * 2001-04-28 fl   added __copy__ methods (work in progress)
 * 2001-05-14 fl   fixes for 1.5.2 compatibility
 * 2001-07-01 fl   added BIGCHARSET support (from Martin von Loewis)
 * 2001-10-18 fl   fixed group reset issue (from Matthew Mueller)
 * 2001-10-20 fl   added split primitive; reenable unicode for 1.6/2.0/2.1
 * 2001-10-21 fl   added sub/subn primitive
 * 2001-10-24 fl   added finditer primitive (for 2.2 only)
 * 2001-12-07 fl   fixed memory leak in sub/subn (Guido van Rossum)
 * 2002-11-09 fl   fixed empty sub/subn return type
 * 2003-04-18 mvl  fully support 4-byte codes
 * 2003-10-17 gn   implemented non recursive scheme
 * 2009-07-26 mrab completely re-designed matcher code
 *
 * Copyright (c) 1997-2001 by Secret Labs AB.  All rights reserved.
 *
 * This version of the SRE library can be redistributed under CNRI's
 * Python 1.6 license.  For any other use, please contact Secret Labs
 * AB (info@pythonware.com).
 *
 * Portions of this engine have been developed in cooperation with
 * CNRI.  Hewlett-Packard provided funding for 1.6 integration and
 * other compatibility work.
 */

#if !defined(RE_RECURSIVE)
/* #define VERBOSE */
#if defined(VERBOSE)
#define TRACE(X) printf X;
#else
#define TRACE(X)
#endif

/* Parsed on first (non-recursive) pass only. */

#include "Python.h"
#include "structmember.h" /* offsetof */
#include <ctype.h>
#include "_regex.h"
#include "pyport.h"

typedef enum {FALSE, TRUE} BOOL;

typedef unsigned int RE_CH;

/* Name of this module, minus the leading underscore. */
#define RE_MODULE "regex"
#define RE_MODULE_UPPER "REGEX"

/* Error codes. */
#define RE_ERROR_SUCCESS 1 /* Successful match. */
#define RE_ERROR_FAILURE 0 /* Unsuccessful match. */
#define RE_ERROR_ILLEGAL -1 /* Illegal opcode. */
#define RE_ERROR_STATE -2 /* Illegal state. */
#define RE_ERROR_MEMORY -9 /* Out of memory. */
#define RE_ERROR_INTERRUPTED -10 /* Signal handler raised exception. */
#define RE_ERROR_REPLACEMENT -11 /* Invalid replacement string. */
#define RE_ERROR_GROUP -12 /* Unknown group. */
#define RE_ERROR_INDEX -13 /* Invalid group index. */

/* Maximum number of iterations at a start position before switching to
 * breadth-wise matching.
 */
#define RE_MAX_ITERATIONS 0x1000

static char copyright[] =
    " RE 2.3.0 Copyright (c) 1997-2002 by Secret Labs AB ";

/* The exception to return on error. */
static PyObject* error_exception;

/* Handlers for ASCII, locale and Unicode. */
typedef struct {
    BOOL (*in_category)(RE_CODE category, RE_CH ch);
    RE_CH (*lower)(RE_CH ch);
    RE_CH (*upper)(RE_CH ch);
    RE_CH (*title)(RE_CH ch);
} RE_EncodingTable;

/* Comparable values in a context struct.
 *
 * Duplicate contexts can be discarded to reduce the amount of work to do.
 */
typedef union {
    struct {
        void* start;
        unsigned int count;
    } repeat;
    struct {
        void* begin;
        void* end;
    } group;
} RE_ContextValue;

/* The context struct itself.
 *
 * Matching a string will produce a doubly-linked list of these. Each ends with
 * entries of RE_ContextValue.
 */
typedef struct RE_Context {
    struct RE_Context* previous;
    struct RE_Context* next;
    struct RE_ContextNode* node;
    void* slice_start;
    void* slice_end;
    void* text_ptr;
    BOOL allow_zero_width;
    int lastindex;
    int lastgroup;
    RE_ContextValue values[1];
} RE_Context;

/* A node as defined in a PatternObject. */
typedef struct RE_PatternNode {
    Py_ssize_t index;
    RE_CODE op;
    int value_capacity;
    int value_count;
    unsigned int* values;
    struct RE_PatternNode* next_1;
    struct RE_PatternNode* next_2;
} RE_PatternNode;

/* A node as used during matching. */
typedef struct RE_ContextNode {
    RE_CODE op;
    unsigned int* values;
    int context_capacity;
    int context_count;
    RE_Context** context_list;
    struct RE_ContextNode* next_1;
    struct RE_ContextNode* next_2;
} RE_ContextNode;

/* Info about a capture group. */
typedef struct {
    int id;
    BOOL referenced;
    BOOL has_name;
    int value_offset;
} RE_GroupInfo;

/* The state object used during matching. */
typedef struct {
    struct PatternObject* pattern; /* Parent PatternObject. */
    /* Info about the string being matched. */
    PyObject* string;
    int charsize;
    RE_EncodingTable* encoding; /* Encoding handlers. */
    void* text_start;
    void* text_end;
    void* slice_start;
    void* slice_end;
    void** marks;
    int lastindex;
    int lastgroup;
    void* text_ptr;
    void* final_newline; /* Pointer to newline at end of string, or NULL. */
    BOOL zero_width; /* Enable the correct handling of zero-width matches. */
    BOOL allow_zero_width; /* Allow a zero-width match to be returned. */
    /* Info about context structs. Discarded ones are retained for reuse. */
    int context_capacity;
    int context_count;
    RE_Context** context_list;
    RE_ContextNode* node_list; /* Context nodes. */
} RE_State;

/* The PatternObject created from a regular expression. */
typedef struct PatternObject {
    PyObject_HEAD
    PyObject* pattern; /* Pattern source (or None). */
    int flags; /* Flags used when compiling pattern source. */
    PyObject* weakreflist; /* List of weak references */
    /* Nodes into which the regular expression is compiled. */
    RE_PatternNode* start_node;
    RE_PatternNode* success_node;
    int repeat_count; /* Number of limited repeats. */
    int group_count; /* Number of capture groups. */
    int ref_group_count; /* Number of groups which are referenced. */
    PyObject* groupindex;
    PyObject* indexgroup;
    /* Storage for the pattern nodes. */
    int node_capacity;
    Py_ssize_t node_count;
    RE_PatternNode** node_list;
    /* Info about the capture groups. */
    Py_ssize_t group_info_capacity;
    RE_GroupInfo* group_info_list;
    /* Info about the context structs. */
    int context_size; /* Size of context struct. */
    int compare_size; /* Total size of the values for comparing contexts. */
} PatternObject;

/* The MatchObject created when a match is found. */
typedef struct {
    PyObject_HEAD
    PyObject* string; /* Link to the target string. */
    PatternObject* pattern; /* Link to the regex (pattern) object. */
    Py_ssize_t pos; /* Start of current slice. */
    Py_ssize_t endpos; /* End of current slice. */
    int lastindex; /* Last group seen by the engine (-1 if none). */
    int lastgroup; /* Last named group seen by the engine (-1 if none). */
    int group_count;
    Py_ssize_t* marks;
    PyObject* regs;
} MatchObject;

/* The ScannerObject. */
typedef struct {
    PyObject_HEAD
    PyObject* pattern;
    RE_State state;
} ScannerObject;

/* ASCII-specific. */

#define RE_ASCII_MAX 0x7F

/* Checks whether an ASCII character is in the given category. */
static BOOL ascii_in_category(RE_CODE category, RE_CH ch) {
    if (ch > RE_ASCII_MAX)
        /* Outside the ASCII range. */
        return FALSE;

    switch (category) {
    case RE_CAT_ALNUM:
        return (re_ascii_category[ch] & RE_MASK_ALNUM) != 0;
    case RE_CAT_ALPHA:
        return (re_ascii_category[ch] & RE_MASK_ALPHA) != 0;
    case RE_CAT_ASCII:
        return TRUE;
    case RE_CAT_BLANK:
        return ch == '\t' || ch == ' ';
    case RE_CAT_CNTRL:
        return ch < 0x20 || ch == 0x7F;
    case RE_CAT_DIGIT:
        return (re_ascii_category[ch] & RE_MASK_DIGIT) != 0;
    case RE_CAT_GRAPH:
        return 0x21 <= ch && ch <= 0x7E;
    case RE_CAT_LINEBREAK:
        return ch == '\n';
    case RE_CAT_LOWER:
        return (re_ascii_category[ch] & RE_MASK_LOWER) != 0;
    case RE_CAT_PRINT:
        return 0x20 <= ch && ch <= 0x7E;
    case RE_CAT_PUNCT:
        return (re_ascii_category[ch] & RE_MASK_PUNCT) != 0;
    case RE_CAT_SPACE:
        return (re_ascii_category[ch] & RE_MASK_SPACE) != 0;
    case RE_CAT_UPPER:
        return (re_ascii_category[ch] & RE_MASK_UPPER) != 0;
    case RE_CAT_WORD:
        return ch == '_' || (re_ascii_category[ch] & RE_MASK_ALNUM) != 0;
    case RE_CAT_XDIGIT:
        return (re_ascii_category[ch] & RE_MASK_XDIGIT) != 0;
    default:
        return FALSE;
    }
}

/* Converts an ASCII character to lowercase. */
static RE_CH ascii_lower(RE_CH ch) {
    if (ch <= RE_ASCII_MAX && (re_ascii_category[ch] & RE_MASK_UPPER) != 0)
        return ch ^ 0x20;

    return ch;
}

/* Converts an ASCII character to uppercase. */
static RE_CH ascii_upper(RE_CH ch) {
    if (ch <= RE_ASCII_MAX && (re_ascii_category[ch] & RE_MASK_LOWER) != 0)
        return ch ^ 0x20;

    return ch;
}

/* The handlers for ASCII characters. */
static RE_EncodingTable ascii_encoding = {
    ascii_in_category,
    ascii_lower,
    ascii_upper,
    ascii_upper, /* Titlecase for ASCII is the same as uppercase. */
};

/* Locale-specific. */

#define RE_LOCALE_MAX 0xFF

/* Checks whether a locale character is in the given category. */
static BOOL locale_in_category(RE_CODE category, RE_CH ch) {
    if (ch > RE_LOCALE_MAX)
        /* Outside the locale range. */
        return FALSE;

    switch (category) {
    case RE_CAT_ALNUM:
        return isalnum(ch) != 0;
    case RE_CAT_ALPHA:
        return isalpha(ch) != 0;
    case RE_CAT_ASCII:
        return ch <= RE_ASCII_MAX;
    case RE_CAT_BLANK:
        return ch == '\t' || ch == ' ';
    case RE_CAT_CNTRL:
        return iscntrl(ch) != 0;
    case RE_CAT_DIGIT:
        return isdigit(ch) != 0;
    case RE_CAT_GRAPH:
        return isgraph(ch) != 0;
    case RE_CAT_LINEBREAK:
        return ch == '\n';
    case RE_CAT_LOWER:
        return islower(ch) != 0;
    case RE_CAT_PRINT:
        return isprint(ch) != 0;
    case RE_CAT_PUNCT:
        return ispunct(ch) != 0;
    case RE_CAT_SPACE:
        return isspace(ch) != 0;
    case RE_CAT_UPPER:
        return isupper(ch) != 0;
    case RE_CAT_WORD:
        return ch == '_' || isalnum(ch) != 0;
    case RE_CAT_XDIGIT:
        return ch <= RE_ASCII_MAX && (re_ascii_category[ch] & RE_MASK_XDIGIT) !=
          0;
    default:
        return FALSE;
    }
}

/* Converts a locale character to lowercase. */
static RE_CH locale_lower(RE_CH ch) {
    if (ch <= RE_LOCALE_MAX)
        return tolower(ch);

    return ch;
}

/* Converts a locale character to uppercase. */
static RE_CH locale_upper(RE_CH ch) {
    if (ch <= RE_LOCALE_MAX)
        return toupper(ch);

    return ch;
}

/* The handlers for locale characters. */
static RE_EncodingTable locale_encoding = {
    locale_in_category,
    locale_lower,
    locale_upper,
    locale_upper, /* Titlecase for locale is the same as uppercase. */
};

/* Unicode-specific. */

/* Unicode character properties. */

/* (Typedefs copied from unicodedata.c) */

#if PY_VERSION_HEX < 0x02060000
typedef struct {
    const unsigned char category;	/* index into
					   _PyUnicode_CategoryNames */
    const unsigned char	combining; 	/* combining class value 0 - 255 */
    const unsigned char	bidirectional; 	/* index into
					   _PyUnicode_BidirectionalNames */
    const unsigned char mirrored;	/* true if mirrored in bidir mode */
    const unsigned char east_asian_width;	/* index into
						   _PyUnicode_EastAsianWidth */
} _PyUnicode_DatabaseRecord;
#else
typedef struct {
    const unsigned char category;   /* index into
                       _PyUnicode_CategoryNames */
    const unsigned char combining;  /* combining class value 0 - 255 */
    const unsigned char bidirectional;  /* index into
                       _PyUnicode_BidirectionalNames */
    const unsigned char mirrored;   /* true if mirrored in bidir mode */
    const unsigned char east_asian_width;   /* index into
                           _PyUnicode_EastAsianWidth */
    const unsigned char normalization_quick_check; /* see is_normalized() */
} _PyUnicode_DatabaseRecord;
#endif

typedef struct change_record {
    /* sequence of fields should be the same as in merge_old_version */
    const unsigned char bidir_changed;
    const unsigned char category_changed;
    const unsigned char decimal_changed;
    const unsigned char mirrored_changed;
    const int numeric_changed;
} change_record;

/* data file generated by Tools/unicode/makeunicodedata.py */
#include "unicodedata_db.h"

static const _PyUnicode_DatabaseRecord*
_getrecord_ex(Py_UCS4 code)
{
    int index;
    if (code >= 0x110000)
        index = 0;
    else {
        index = index1[(code>>SHIFT)];
        index = index2[(index<<SHIFT)+(code&((1<<SHIFT)-1))];
    }

    return &_PyUnicode_Database_Records[index];
}

/* Checks whether a Unicode character is in the given category. */
static BOOL unicode_in_category(RE_CODE category, RE_CH ch) {
    unsigned int flag;

    switch (category) {
    case RE_CAT_ALNUM:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_ALNUM) != 0;
    case RE_CAT_ALPHA:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_L) != 0;
    case RE_CAT_ASCII:
        return ch <= RE_ASCII_MAX;
    case RE_CAT_BLANK:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_ZS) != 0 || ch == '\t';
    case RE_CAT_C:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_C) != 0;
    case RE_CAT_CC:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_CC) != 0;
    case RE_CAT_CF:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_CF) != 0;
    case RE_CAT_CN:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_CN) != 0;
    case RE_CAT_CNTRL:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_CC) != 0;
    case RE_CAT_CO:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_CO) != 0;
    case RE_CAT_CS:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_CS) != 0;
    case RE_CAT_DIGIT:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_ND) != 0;
    case RE_CAT_GRAPH:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_NONGRAPH) == 0;
    case RE_CAT_L:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_L) != 0;
    case RE_CAT_LINEBREAK:
        return ch == '\n';
    case RE_CAT_LL:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_LL) != 0;
    case RE_CAT_LM:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_LM) != 0;
    case RE_CAT_LO:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_LO) != 0;
    case RE_CAT_LOWER:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_LL) != 0;
    case RE_CAT_LT:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_LT) != 0;
    case RE_CAT_LU:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_LU) != 0;
    case RE_CAT_M:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_M) != 0;
    case RE_CAT_MC:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_MC) != 0;
    case RE_CAT_ME:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_ME) != 0;
    case RE_CAT_MN:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_MN) != 0;
    case RE_CAT_N:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_N) != 0;
    case RE_CAT_ND:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_ND) != 0;
    case RE_CAT_NL:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_NL) != 0;
    case RE_CAT_NO:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_NO) != 0;
    case RE_CAT_P:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_P) != 0;
    case RE_CAT_PC:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_PC) != 0;
    case RE_CAT_PD:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_PD) != 0;
    case RE_CAT_PE:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_PE) != 0;
    case RE_CAT_PF:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_PF) != 0;
    case RE_CAT_PI:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_PI) != 0;
    case RE_CAT_PO:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_PO) != 0;
    case RE_CAT_PRINT:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_C) == 0;
    case RE_CAT_PS:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_PS) != 0;
    case RE_CAT_PUNCT:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_PUNCT) != 0;
    case RE_CAT_S:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_S) != 0;
    case RE_CAT_SC:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_SC) != 0;
    case RE_CAT_SK:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_SK) != 0;
    case RE_CAT_SM:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_SM) != 0;
    case RE_CAT_SO:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_SO) != 0;
    case RE_CAT_SPACE:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_Z) != 0 || ch == '\t' || ch == '\n' || ch ==
          '\f' || ch == '\r' || ch == '\v';
    case RE_CAT_UPPER:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_LU) != 0;
    case RE_CAT_WORD:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_WORD) != 0;
    case RE_CAT_XDIGIT:
        return ch <= RE_ASCII_MAX && (re_ascii_category[ch] & RE_MASK_XDIGIT) !=
          0;
    case RE_CAT_Z:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_Z) != 0;
    case RE_CAT_ZL:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_ZL) != 0;
    case RE_CAT_ZP:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_ZP) != 0;
    case RE_CAT_ZS:
        flag = 1 << _getrecord_ex((Py_UCS4)ch)->category;
        return (flag & RE_PROP_MASK_ZS) != 0;
    default:
        return FALSE;
    }
}

/* Converts a Unicode character to lowercase. */
static RE_CH unicode_lower(RE_CH ch) {
    return Py_UNICODE_TOLOWER((Py_UNICODE)ch);
}

/* Converts a Unicode character to uppercase. */
static RE_CH unicode_upper(RE_CH ch) {
    return Py_UNICODE_TOUPPER((Py_UNICODE)ch);
}

/* Converts a Unicode character to titlecase. */
static RE_CH unicode_title(RE_CH ch) {
    return Py_UNICODE_TOTITLE((Py_UNICODE)ch);
}

/* The handlers for Unicode characters. */
static RE_EncodingTable unicode_encoding = {
    unicode_in_category,
    unicode_lower,
    unicode_upper,
    unicode_title,
};

/* Sets the error message. */
Py_LOCAL(void) set_error(int status, PyObject* object) {
    TRACE(("<<set_error>>\n"))
    switch (status) {
    case RE_ERROR_GROUP:
        PyErr_SetString(PyExc_IndexError, "unknown group");
        break;
    case RE_ERROR_ILLEGAL:
        PyErr_SetString(PyExc_RuntimeError, "invalid RE code");
        break;
    case RE_ERROR_INTERRUPTED:
        /* An exception has already been raised, so let it fly. */
        break;
    case RE_ERROR_MEMORY:
        PyErr_NoMemory();
        break;
    case RE_ERROR_REPLACEMENT:
        PyErr_SetString(error_exception, "invalid replacement");
        break;
    case RE_ERROR_INDEX:
        if (object)
            PyErr_Format(PyExc_TypeError,
              "group indices must be integers or strings, not %.200s",
              object->ob_type->tp_name);
        else
            PyErr_Format(PyExc_TypeError,
              "group indices must be integers or strings");
        break;
    default:
        /* Other error codes indicate compiler/engine bugs. */
        PyErr_SetString(PyExc_RuntimeError,
          "internal error in regular expression engine");
    }
}

/* Allocates a context.
 *
 * Where available, the context is taken from the array of discarded contexts.
 */
Py_LOCAL(RE_Context*) alloc_context(RE_State* state) {
    RE_Context* context;

    if (state->context_count > 0)
        context = state->context_list[--state->context_count];
    else {
        context = (RE_Context*)PyMem_MALLOC(state->pattern->context_size);
        if (!context) {
            set_error(RE_ERROR_MEMORY, NULL);
        }
    }

    return context;
}

/* Frees a context.
 *
 * The context is stored for future reuse in an array.
 */
Py_LOCAL(void) free_context(RE_State* state, RE_Context* context) {
    if (state->context_count >= state->context_capacity) {
        int new_capacity;
        RE_Context** new_list;

        new_capacity = state->context_capacity * 2;
        new_list = (RE_Context**)PyMem_REALLOC(state->context_list, new_capacity
          * sizeof(*state->context_list));
        if (!new_list) {
            /* Out of memory? Just free the context. */
            PyMem_FREE(context);
            return;
        }
        state->context_capacity = new_capacity;
        state->context_list = new_list;
    }

    state->context_list[state->context_count++] = context;
}

/* Creates the base context.
 *
 * The base context provides the template from which to create new contexts
 * located at the start node.
 */
Py_LOCAL(RE_Context*) create_base_context(RE_State* state) {
    RE_Context* context;

    context = alloc_context(state);
    if (!context)
        return NULL;

    memset(context, 0, state->pattern->context_size);
    context->previous = context;
    context->next = context;
    context->node = &state->node_list[0];
    context->slice_start = state->slice_start;
    context->slice_end = state->slice_end;
    context->text_ptr = state->slice_start;
    context->allow_zero_width = state->allow_zero_width;
    context->lastindex = -1;
    context->lastgroup = -1;

    return context;
}

/* Copies a context, inserting the copy before the original. */
Py_LOCAL(RE_Context*) copy_context_before(RE_State* state, RE_Context* context)
  {
    RE_Context* copy;
    RE_Context* previous;

    copy = alloc_context(state);
    if (!copy)
        return NULL;

    memmove(copy, context, state->pattern->context_size);
    previous = context->previous;
    previous->next = copy;
    copy->next = context;
    context->previous = copy;

    return copy;
}

/* Copies a context, inserting the copy after the original. */
Py_LOCAL(RE_Context*) copy_context_after(RE_State* state, RE_Context* context) {
    RE_Context* copy;
    RE_Context* next;

    copy = alloc_context(state);
    if (!copy)
        return NULL;

    memmove(copy, context, state->pattern->context_size);
    next = copy->next;
    context->next = copy;
    next->previous = copy;
    copy->previous = context;

    return copy;
}

/* Frees all the contexts. */
Py_LOCAL(void) free_all_contexts(RE_State* state, RE_Context* base_context) {
    RE_Context* context;

    context = base_context->next;
    while (context != base_context) {
        RE_Context* next;

        next = context->next;
        free_context(state, context);
        context = next;
    }

    free_context(state, base_context);
}

/* Deletes a context from the context list. */
Py_LOCAL(void) delete_context(RE_State* state, RE_Context* context) {
    RE_Context* previous;
    RE_Context* next;

    previous = context->previous;
    next = context->next;
    previous->next = next;
    next->previous = previous;

    free_context(state, context);
}

/* Rejects the context.
 *
 * The context is deleted from the list and the next one is returned.
 */
Py_LOCAL(RE_Context*) reject_context(RE_State* state, RE_Context* context) {
    RE_Context* next;

    next = context->next;
    delete_context(state, context);

    return next;
}

/* Checks whether 2 contexts are the same (actually, have the same comparable
 * values).
 */
Py_LOCAL(BOOL) same_context(RE_State* state, RE_Context* context_1, RE_Context*
  context_2) {
    return context_1->text_ptr == context_2->text_ptr &&
      memcmp(context_1->values, context_2->values, state->pattern->compare_size)
      == 0;
}

/* Checks whether a context can be merged with another context already seen. */
Py_LOCAL(BOOL) can_merge_contexts(RE_State* state, RE_Context* context) {
    RE_ContextNode* node;
    int i;

    node = context->node;
    for (i = 0; i < node->context_count; i++) {
        if (same_context(state, context, node->context_list[i]))
            return TRUE;
    }

    return FALSE;
}

/* Adds a context to a context node. */
Py_LOCAL(BOOL) add_context(RE_ContextNode* node, RE_Context* context) {
    if (node->context_count >= node->context_capacity) {
        RE_Context** new_list;

        node->context_capacity *= 2;
        new_list = (RE_Context**)PyMem_REALLOC(node->context_list,
          node->context_capacity * sizeof(*node->context_list));
        if (!new_list) {
            set_error(RE_ERROR_MEMORY, NULL);
            return FALSE;
        }
        node->context_list = new_list;
    }

    node->context_list[node->context_count++] = context;

    return TRUE;
}

/* Checks whether a character is in a set. */
Py_LOCAL(BOOL) in_set(RE_State* state, unsigned int* values, RE_CH ch) {
    BOOL (*in_category)(RE_CODE, RE_CH);
    int p;

    in_category = state->encoding->in_category;

    /* Element 0 is the match flag. */
    p = 1;
    for (;;) {
        switch (values[p]) {
        case RE_OP_CATEGORY:
            TRACE(("RE_OP_CATEGORY %d %d\n", values[p + 1], values[p + 2]))
            if (in_category(values[p + 2], ch) == values[p + 1])
                return TRUE;
            p += 3;
            break;
        case RE_OP_CHARACTER:
            TRACE(("RE_OP_CHARACTER %d\n", values[p + 1]))
            if (ch == values[p + 1])
                return TRUE;
            p += 2;
            break;
        case RE_OP_RANGE:
            TRACE(("RE_OP_RANGE %d %d\n", values[p + 1], values[p + 2]))
            if (ch >= values[p + 1] && ch <= values[p + 2])
                return TRUE;
            p += 3;
            break;
        default:
            /* Should be 'END'. */
            return FALSE;
        }
    }
}

/* Checks whether 2 characters are the same, ignoring case. */
Py_LOCAL(BOOL) same_char_ignore(RE_EncodingTable* encoding, RE_CH ch_1, RE_CH
  ch_2) {
    return encoding->lower(ch_1) == encoding->lower(ch_2) ||
      encoding->upper(ch_1) == encoding->upper(ch_2) || encoding->title(ch_1) ==
      encoding->title(ch_2);
}

/* Checks whether a character is in a set, ignoring case. */
Py_LOCAL(BOOL) in_set_ignore(RE_State* state, unsigned int* values, RE_CH ch) {
    RE_EncodingTable* encoding;
    BOOL (*in_category)(RE_CODE, RE_CH);
    int p;

    encoding = state->encoding;
    in_category = encoding->in_category;

    /* Element 0 is the match flag. */
    p = 1;
    for (;;) {
        switch (values[p]) {
        case RE_OP_CATEGORY:
            TRACE(("RE_OP_CATEGORY %d %d\n", values[p + 1], values[p + 2]))
            if (in_category(values[p + 2], ch) == values[p + 1])
                return TRUE;
            p += 3;
            break;
        case RE_OP_CHARACTER:
            TRACE(("RE_OP_CHARACTER %d\n", values[p + 1]))
            if (same_char_ignore(encoding, ch, values[p + 1]))
                return TRUE;
            p += 2;
            break;
        case RE_OP_RANGE:
        {
            RE_CH c;
            TRACE(("RE_OP_RANGE %d %d\n", values[p + 1], values[p +
              2]))

            c = encoding->lower(ch);
            if (c >= values[p + 1] && c <= values[p + 2])
                return TRUE;
            c = encoding->upper(ch);
            if (c >= values[p + 1] && c <= values[p + 2])
                return TRUE;
            c = encoding->title(ch);
            if (c >= values[p + 1] && c <= values[p + 2])
                return TRUE;
            p += 3;
            break;
        }
        default:
            /* Should be 'END'. */
            return FALSE;
        }
    }
}

/* Gets the integer offset into the string.
 *
 * Returns -1 if the text pointer is NULL.
 */
Py_LOCAL(Py_ssize_t) text_offset(RE_State* state, void* ptr) {
    return ptr < state->text_start ? -1 : ((char*)ptr -
      (char*)state->text_start) / state->charsize;
}

/* Set up for byte-string. */
#define RE_CHAR unsigned char
#define RE_AT_BOUNDARY bat_boundary
#define RE_TRY_MATCH btry_match
#define RE_MATCH_CONTEXT bmatch_context
#define RE_MATCH bmatch
#endif

/* Parsed on both passes (for byte-string and Unicode). */

/* Checks whether the current text position is on a word boundary. */
Py_LOCAL(BOOL) RE_AT_BOUNDARY(RE_State* state, RE_Context* context) {
    BOOL (*in_category)(RE_CODE, RE_CH);
    RE_CHAR* text_ptr;
    BOOL before;
    BOOL after;

    in_category = state->encoding->in_category;
    text_ptr = context->text_ptr;

    before = text_ptr > (RE_CHAR*)state->text_start && in_category(RE_CAT_WORD,
      text_ptr[-1]);
    after = text_ptr < (RE_CHAR*)state->text_end && in_category(RE_CAT_WORD,
      text_ptr[0]);

    return before != after;
}

/* Tries a match at the current text position.
 *
 * Returns TRUE and the updated node and text position if the match succeeds.
 */
Py_LOCAL(BOOL) RE_TRY_MATCH(RE_ContextNode* node, RE_Context* context, RE_State*
  state, RE_ContextNode** next_node, RE_CHAR** next_text_ptr) {
    RE_CHAR* text_ptr;
    RE_ContextNode* test_node;

    text_ptr = context->text_ptr;

    test_node = node;
    while (test_node->op == RE_OP_BEGIN_GROUP || test_node->op ==
      RE_OP_END_GROUP)
        test_node = test_node->next_1;

    switch (test_node->op) {
    case RE_OP_ANY: /* Any character, except a newline. */
        if (text_ptr >= (RE_CHAR*)context->slice_end || text_ptr[0] == '\n')
            return FALSE;

        if (test_node == node) {
            *next_node = node->next_1;
            *next_text_ptr = text_ptr + 1;
        } else {
            *next_node = node;
            *next_text_ptr = text_ptr;
        }
        return TRUE;
    case RE_OP_ANY_ALL: /* Any character at all. */
        if (text_ptr >= (RE_CHAR*)context->slice_end)
            return FALSE;

        if (test_node == node) {
            *next_node = node->next_1;
            *next_text_ptr = text_ptr + 1;
        } else {
            *next_node = node;
            *next_text_ptr = text_ptr;
        }
        return TRUE;
    case RE_OP_ANY_ALL_REV: /* Any character at all. */
        if (text_ptr < (RE_CHAR*)context->slice_start)
            return FALSE;

        if (test_node == node) {
            *next_node = node->next_1;
            *next_text_ptr = text_ptr - 1;
        } else {
            *next_node = node;
            *next_text_ptr = text_ptr;
        }
        return TRUE;
    case RE_OP_ANY_REV: /* Any character, except a newline. */
        if (text_ptr < (RE_CHAR*)context->slice_start || text_ptr[-1] == '\n')
            return FALSE;

        if (test_node == node) {
            *next_node = node->next_1;
            *next_text_ptr = text_ptr - 1;
        } else {
            *next_node = node;
            *next_text_ptr = text_ptr;
        }
        return TRUE;
    case RE_OP_BOUNDARY: /* At a word boundary. */
        if (RE_AT_BOUNDARY(state, context) != test_node->values[0])
            return FALSE;

        if (test_node == node)
            *next_node = node->next_1;
        else
            *next_node = node;
        *next_text_ptr = text_ptr;
        return TRUE;
    case RE_OP_CATEGORY: /* A character category. */
    {
        BOOL (*in_category)(RE_CODE, RE_CH);

        in_category = state->encoding->in_category;

        if (text_ptr >= (RE_CHAR*)context->slice_end ||
          in_category(test_node->values[1], text_ptr[0]) !=
          test_node->values[0])
            return FALSE;

        if (test_node == node) {
            *next_node = node->next_1;
            *next_text_ptr = text_ptr + 1;
        } else {
            *next_node = node;
            *next_text_ptr = text_ptr;
        }
        return TRUE;
    }
    case RE_OP_CATEGORY_REV: /* A character category. */
    {
        BOOL (*in_category)(RE_CODE, RE_CH);

        in_category = state->encoding->in_category;

        if (text_ptr < (RE_CHAR*)context->slice_start ||
          in_category(test_node->values[1], text_ptr[-1]) !=
          test_node->values[0])
            return FALSE;

        if (test_node == node) {
            *next_node = node->next_1;
            *next_text_ptr = text_ptr - 1;
        } else {
            *next_node = node;
            *next_text_ptr = text_ptr;
        }
        return TRUE;
    }
    case RE_OP_CHARACTER: /* A character literal. */
        if (text_ptr >= (RE_CHAR*)context->slice_end || text_ptr[0] !=
          test_node->values[0])
            return FALSE;

        if (test_node == node) {
            *next_node = node->next_1;
            *next_text_ptr = text_ptr + 1;
        } else {
            *next_node = node;
            *next_text_ptr = text_ptr;
        }
        return TRUE;
    case RE_OP_CHARACTER_IGNORE: /* A character literal, ignoring case. */
    {
        RE_EncodingTable* encoding;

        encoding = state->encoding;

        if (text_ptr >= (RE_CHAR*)context->slice_end ||
          !same_char_ignore(encoding, text_ptr[0], test_node->values[0]))
            return FALSE;

        if (test_node == node) {
            *next_node = node->next_1;
            *next_text_ptr = text_ptr + 1;
        } else {
            *next_node = node;
            *next_text_ptr = text_ptr;
        }
        return TRUE;
    }
    case RE_OP_CHARACTER_IGNORE_REV: /* A character literal, ignoring case. */
    {
        RE_EncodingTable* encoding;

        encoding = state->encoding;

        if (text_ptr < (RE_CHAR*)context->slice_start ||
          !same_char_ignore(encoding, text_ptr[-1], test_node->values[0]))
            return FALSE;

        if (test_node == node) {
            *next_node = node->next_1;
            *next_text_ptr = text_ptr - 1;
        } else {
            *next_node = node;
            *next_text_ptr = text_ptr;
        }
        return TRUE;
    }
    case RE_OP_CHARACTER_REV: /* A character literal. */
        if (text_ptr < (RE_CHAR*)context->slice_start || text_ptr[-1] !=
          test_node->values[0])
            return FALSE;

        if (test_node == node) {
            *next_node = node->next_1;
            *next_text_ptr = text_ptr - 1;
        } else {
            *next_node = node;
            *next_text_ptr = text_ptr;
        }
        return TRUE;
    case RE_OP_END_OF_LINE: /* At the end of a line. */
        if (text_ptr != state->text_end && text_ptr[0] != '\n')
            return FALSE;

        if (test_node == node)
            *next_node = node->next_1;
        else
            *next_node = node;
        *next_text_ptr = text_ptr;
        return TRUE;
    case RE_OP_END_OF_STRING: /* At the end of the string. */
        if (text_ptr != state->text_end)
            return FALSE;

        if (test_node == node)
            *next_node = node->next_1;
        else
            *next_node = node;
        *next_text_ptr = text_ptr;
        return TRUE;
    case RE_OP_END_OF_STRING_LINE: /* At end of string or final newline. */
        if (text_ptr != state->text_end && text_ptr != state->final_newline)
            return FALSE;

        if (test_node == node)
            *next_node = node->next_1;
        else
            *next_node = node;
        *next_text_ptr = context->text_ptr;
        return TRUE;
    case RE_OP_REF_GROUP: /* Reference to a capture group. */
    {
        int g;
        RE_CHAR* begin_group;
        RE_CHAR* end_group;

        /* Get the offset to the group value in the context. */
        g = test_node->values[0];
        g = state->pattern->group_info_list[g - 1].value_offset;

        /* Check whether the captured text, if any, exists at this position in
         * the string.
         */
        begin_group = context->values[g].group.begin;
        end_group = context->values[g].group.end;
        if ((RE_CHAR*)context->slice_start <= begin_group && begin_group <=
          end_group && end_group <= (RE_CHAR*)context->slice_end) {
            RE_CHAR* text_ptr;
            Py_ssize_t length;
            Py_ssize_t available;

            text_ptr = context->text_ptr;
            length = end_group - begin_group;
            available = (RE_CHAR*)context->slice_end - text_ptr;

            if (length <= available && text_ptr[0] == begin_group[0]) {
                *next_node = node;
                *next_text_ptr = text_ptr;
                return TRUE;
            } else
                return FALSE;
        } else {
            /* The group isn't closed, so assume it could match. */
            *next_node = node;
            *next_text_ptr = text_ptr;
            return TRUE;
        }
    }
    case RE_OP_REF_GROUP_IGNORE: /* Reference to a capture group, ignoring case. */
    {
        int g;
        RE_CHAR* begin_group;
        RE_CHAR* end_group;

        /* Get the offset to the group value in the context. */
        g = test_node->values[0];
        g = state->pattern->group_info_list[g - 1].value_offset;

        /* Check whether the captured text, if any, exists at this position in
         * the string.
         */
        begin_group = context->values[g].group.begin;
        end_group = context->values[g].group.end;
        if ((RE_CHAR*)context->slice_start <= begin_group && begin_group <=
          end_group && end_group <= (RE_CHAR*)context->slice_end) {
            RE_CHAR* text_ptr;
            Py_ssize_t length;
            Py_ssize_t available;
            RE_EncodingTable* encoding;

            text_ptr = context->text_ptr;
            length = end_group - begin_group;
            available = (RE_CHAR*)context->slice_end - text_ptr;

            encoding = state->encoding;

            if (length <= available && same_char_ignore(encoding, text_ptr[0],
              begin_group[0])) {
                *next_node = node;
                *next_text_ptr = text_ptr;
                return TRUE;
            }
        }
        return FALSE;
    }
    case RE_OP_REF_GROUP_IGNORE_REV: /* Reference to a capture group, ignoring case. */
    {
        int g;
        RE_CHAR* begin_group;
        RE_CHAR* end_group;

        /* Get the offset to the group value in the context. */
        g = test_node->values[0];
        g = state->pattern->group_info_list[g - 1].value_offset;

        /* Check whether the captured text, if any, exists at this position in
         * the string.
         */
        begin_group = context->values[g].group.begin;
        end_group = context->values[g].group.end;
        if ((RE_CHAR*)context->slice_start <= begin_group && begin_group <=
          end_group && end_group <= (RE_CHAR*)context->slice_end) {
            RE_CHAR* text_ptr;
            Py_ssize_t length;
            Py_ssize_t available;
            RE_EncodingTable* encoding;

            text_ptr = context->text_ptr;
            length = end_group - begin_group;
            available = text_ptr - (RE_CHAR*)context->slice_start;

            encoding = state->encoding;

            if (length <= available && same_char_ignore(encoding, text_ptr[-1],
              end_group[-1])) {
                *next_node = node;
                *next_text_ptr = text_ptr;
                return TRUE;
            }
        }
        return FALSE;
    }
    case RE_OP_REF_GROUP_REV: /* Reference to a capture group. */
    {
        int g;
        RE_CHAR* begin_group;
        RE_CHAR* end_group;

        /* Get the offset to the group value in the context. */
        g = test_node->values[0];
        g = state->pattern->group_info_list[g - 1].value_offset;

        /* Check whether the captured text, if any, exists at this position in
         * the string.
         */
        begin_group = context->values[g].group.begin;
        end_group = context->values[g].group.end;
        if ((RE_CHAR*)context->slice_start <= begin_group && begin_group <=
          end_group && end_group <= (RE_CHAR*)context->slice_end) {
            RE_CHAR* text_ptr;
            Py_ssize_t length;
            Py_ssize_t available;

            text_ptr = context->text_ptr;
            length = end_group - begin_group;
            available = text_ptr - (RE_CHAR*)context->slice_start;

            if (length <= available && text_ptr[-1] == end_group[-1]) {
                *next_node = node;
                *next_text_ptr = text_ptr;
                return TRUE;
            } else
                return FALSE;
        } else {
            /* The group isn't closed, so assume it could match. */
            *next_node = node;
            *next_text_ptr = text_ptr;
            return TRUE;
        }
    }
    case RE_OP_SET: /* Character set. */
        if (text_ptr >= (RE_CHAR*)context->slice_end || in_set(state,
          test_node->values, text_ptr[0]) != test_node->values[0])
            return FALSE;

        if (test_node == node) {
            *next_node = node->next_1;
            *next_text_ptr = text_ptr + 1;
        } else {
            *next_node = node;
            *next_text_ptr = text_ptr;
        }
        return TRUE;
    case RE_OP_SET_IGNORE: /* Character set, ignoring case. */
        if (text_ptr >= (RE_CHAR*)context->slice_end || in_set_ignore(state,
          test_node->values, text_ptr[0]) != test_node->values[0])
            return FALSE;

        if (test_node == node) {
            *next_node = node->next_1;
            *next_text_ptr = text_ptr + 1;
        } else {
            *next_node = node;
            *next_text_ptr = text_ptr;
        }
        return TRUE;
    case RE_OP_SET_IGNORE_REV: /* Character set, ignoring case. */
        if (text_ptr < (RE_CHAR*)context->slice_start || in_set_ignore(state,
          test_node->values, text_ptr[-1]) != test_node->values[0])
            return FALSE;

        if (test_node == node) {
            *next_node = node->next_1;
            *next_text_ptr = text_ptr - 1;
        } else {
            *next_node = node;
            *next_text_ptr = text_ptr;
        }
        return TRUE;
    case RE_OP_SET_REV: /* Character set. */
        if (text_ptr < (RE_CHAR*)context->slice_start || in_set(state,
          test_node->values, text_ptr[-1]) != test_node->values[0])
            return FALSE;

        if (test_node == node) {
            *next_node = node->next_1;
            *next_text_ptr = text_ptr - 1;
        } else {
            *next_node = node;
            *next_text_ptr = text_ptr;
        }
        return TRUE;
    case RE_OP_START_OF_LINE: /* At the start of a line. */
        if (text_ptr != state->text_start && text_ptr[-1] != '\n')
            return FALSE;

        if (test_node == node)
            *next_node = node->next_1;
        else
            *next_node = node;
        *next_text_ptr = context->text_ptr;
        return TRUE;
    case RE_OP_START_OF_STRING: /* At the start of the string. */
        if (text_ptr != state->text_start)
            return FALSE;

        if (test_node == node)
            *next_node = node->next_1;
        else
            *next_node = node;
        *next_text_ptr = text_ptr;
        return TRUE;
    default:
        /* Anything else is a possible match. */
        *next_node = node;
        *next_text_ptr = text_ptr;
        return TRUE;
    }
}

Py_LOCAL(int) RE_MATCH_CONTEXT(RE_State* state, RE_Context* base_context, BOOL
  search, BOOL lookahead) {
    RE_Context* context;
    BOOL depth_first;
    int iterations;
    int start_iterations;
    RE_EncodingTable* encoding;
    BOOL (*in_category)(RE_CODE, RE_CH);
    Py_ssize_t node_count;
    RE_ContextNode* node_list;
    RE_GroupInfo* group_info_list;
    RE_CHAR* text_start;
    RE_CHAR* text_end;
    TRACE(("<<RE_MATCH_CONTEXT>>\n"))

    /* Create the first work context. */
    context = copy_context_before(state, base_context);
    if (!context)
        return RE_ERROR_MEMORY;

    /* We initially work depth-first so that well-written regular expressions
     * match quickly. If we're still looking after RE_MAX_ITERATIONS iterations
     * then we conclude that we have one of those 'slow' regular expressions and
     * switch to working breadth-first instead.
     */
    depth_first = TRUE;
    iterations = 0;
    start_iterations = 0;

    encoding = state->encoding;
    in_category = encoding->in_category;

    node_count = state->pattern->node_count;
    node_list = state->node_list;
    group_info_list = state->pattern->group_info_list;
    text_start = state->text_start;
    text_end = state->text_end;

    for (;;) {
        RE_ContextNode* node;
        RE_CHAR* text_ptr;

        node = context->node;
        text_ptr = context->text_ptr;

        TRACE(("%d|", text_ptr - text_start))

        switch (node->op) {
        case RE_OP_ANY: /* Any character, except a newline. */
            TRACE(("%s\n", re_op_text[node->op]))
            if (text_ptr < (RE_CHAR*)context->slice_end && text_ptr[0] != '\n')
              {
                context->node = node->next_1;
                ++(RE_CHAR*)context->text_ptr;
            } else
                context = reject_context(state, context);
            break;
        case RE_OP_ANY_ALL: /* Any character at all. */
            TRACE(("%s\n", re_op_text[node->op]))
            if (text_ptr < (RE_CHAR*)context->slice_end) {
                context->node = node->next_1;
                ++(RE_CHAR*)context->text_ptr;
            } else
                context = reject_context(state, context);
            break;
        case RE_OP_ANY_ALL_REV: /* Any character at all. */
            TRACE(("%s\n", re_op_text[node->op]))
            if (text_ptr > (RE_CHAR*)context->slice_start) {
                context->node = node->next_1;
                --(RE_CHAR*)context->text_ptr;
            } else
                context = reject_context(state, context);
            break;
        case RE_OP_ANY_REV: /* Any character, except a newline. */
            TRACE(("%s\n", re_op_text[node->op]))
            if (text_ptr > (RE_CHAR*)context->slice_start && text_ptr[-1] !=
              '\n') {
                context->node = node->next_1;
                --(RE_CHAR*)context->text_ptr;
            } else
                context = reject_context(state, context);
            break;
        case RE_OP_ATOMIC: /* Atomic subpattern. */
        {
            RE_Context* previous;
            RE_Context* next;
            int status;
            TRACE(("%s\n", re_op_text[node->op]))

            /* Isolate the context so that it can act as a base context. */
            previous = context->previous;
            next = context->next;
            context->previous = context;
            context->next = context;

            /* Point the context to the subpattern and try to match. */
            context->node = node->next_2;
            status = RE_MATCH_CONTEXT(state, context, FALSE, FALSE);

            /* Re-insert the context. */
            context->previous = previous;
            context->next = next;

            if (status < 0)
                return status;

            if (status == RE_ERROR_SUCCESS)
                context->node = node->next_1;
            else
                context = reject_context(state, context);
            break;
        }
        case RE_OP_BEGIN_GROUP: /* Beginning of capture group. */
        {
            int g;
            RE_GroupInfo* info;
            TRACE(("%s %d\n", re_op_text[node->op], node->values[0]))

            /* Group id. */
            g = node->values[0];
            context->lastindex = g;

            info = &group_info_list[g - 1];
            if (info->has_name)
                context->lastgroup = g;

            g = info->value_offset;
            context->values[g].group.begin = text_ptr;

            context->node = node->next_1;
            break;
        }
        case RE_OP_BOUNDARY: /* At a word boundary. */
            TRACE(("%s %d\n", re_op_text[node->op], node->values[0]))
            if (RE_AT_BOUNDARY(state, context) == node->values[0])
                context->node = node->next_1;
            else
                context = reject_context(state, context);
            break;
        case RE_OP_BRANCH: /* 2-way branch. */
        {
            BOOL branch_1;
            RE_ContextNode* next_node_1;
            RE_CHAR* text_ptr_1;
            BOOL branch_2;
            RE_ContextNode* next_node_2;
            RE_CHAR* text_ptr_2;
            TRACE(("%s\n", re_op_text[node->op]))

            /* Test both branches for possible matches.
             *
             * If both branches could match, then the first takes precedence.
             */
            branch_1 = RE_TRY_MATCH(node->next_1, context, state, &next_node_1,
              &text_ptr_1);
            branch_2 = RE_TRY_MATCH(node->next_2, context, state, &next_node_2,
              &text_ptr_2);
            if (branch_1) {
                if (branch_2) {
                    RE_Context* alt_context;

                    alt_context = copy_context_after(state, context);
                    alt_context->node = next_node_2;
                    alt_context->text_ptr = text_ptr_2;
                }
                context->node = next_node_1;
                context->text_ptr = text_ptr_1;
            } else {
                if (branch_2) {
                    context->node = next_node_2;
                    context->text_ptr = text_ptr_2;
                } else
                    context = reject_context(state, context);
            }
            break;
        }
        case RE_OP_CATEGORY: /* A character category. */
        {
            TRACE(("%s %d %d\n", re_op_text[node->op], node->values[0],
              node->values[1]))

            if (text_ptr < (RE_CHAR*)context->slice_end &&
              in_category(node->values[1], text_ptr[0]) == node->values[0]) {
                context->node = node->next_1;
                ++(RE_CHAR*)context->text_ptr;
            } else
                context = reject_context(state, context);
            break;
        }
        case RE_OP_CATEGORY_REV: /* A character category. */
        {
            TRACE(("%s %d %d\n", re_op_text[node->op], node->values[0],
              node->values[1]))

            if (text_ptr > (RE_CHAR*)context->slice_start &&
              in_category(node->values[1], text_ptr[-1]) == node->values[0]) {
                context->node = node->next_1;
                --(RE_CHAR*)context->text_ptr;
            } else
                context = reject_context(state, context);
            break;
        }
        case RE_OP_CHARACTER: /* A character literal. */
            TRACE(("%s %d\n", re_op_text[node->op], node->values[0]))
            if (text_ptr < (RE_CHAR*)context->slice_end && text_ptr[0] ==
              node->values[0]) {
                context->node = node->next_1;
                ++(RE_CHAR*)context->text_ptr;
            } else
                context = reject_context(state, context);
            break;
        case RE_OP_CHARACTER_IGNORE: /* A character literal, ignoring case. */
            TRACE(("%s %d\n", re_op_text[node->op], node->values[0]))
            if (text_ptr < (RE_CHAR*)context->slice_end &&
              same_char_ignore(encoding, text_ptr[0], node->values[0])) {
                context->node = node->next_1;
                ++(RE_CHAR*)context->text_ptr;
            } else
                context = reject_context(state, context);
            break;
        case RE_OP_CHARACTER_IGNORE_REV: /* A character literal, ignoring case. */
            TRACE(("%s %d\n", re_op_text[node->op], node->values[0]))
            if (text_ptr > (RE_CHAR*)context->slice_start &&
              same_char_ignore(encoding, text_ptr[-1], node->values[0])) {
                context->node = node->next_1;
                --(RE_CHAR*)context->text_ptr;
            } else
                context = reject_context(state, context);
            break;
        case RE_OP_CHARACTER_REV: /* A character literal. */
            TRACE(("%s %d\n", re_op_text[node->op], node->values[0]))
            if (text_ptr > (RE_CHAR*)context->slice_start && text_ptr[-1] ==
              node->values[0]) {
                context->node = node->next_1;
                --(RE_CHAR*)context->text_ptr;
            } else
                context = reject_context(state, context);
            break;
        case RE_OP_END_GROUP: /* End of a capture group. */
        {
            int g;
            RE_GroupInfo* info;
            TRACE(("%s %d\n", re_op_text[node->op], node->values[0]))

            /* Group id. */
            g = node->values[0];
            context->lastindex = g;

            info = &group_info_list[g - 1];
            if (info->has_name)
                context->lastgroup = g;

            g = info->value_offset;
            context->values[g].group.end = text_ptr;

            context->node = node->next_1;
            break;
        }
        case RE_OP_END_OF_LINE: /* At the end of a line. */
            TRACE(("%s\n", re_op_text[node->op]))
            if (text_ptr == text_end || text_ptr[0] == '\n')
                context->node = node->next_1;
            else
                context = reject_context(state, context);
            break;
        case RE_OP_END_OF_STRING: /* At the end of the string. */
            TRACE(("%s\n", re_op_text[node->op]))
            if (text_ptr == text_end)
                context->node = node->next_1;
            else
                context = reject_context(state, context);
            break;
        case RE_OP_END_OF_STRING_LINE: /* At end of string or final newline. */
            TRACE(("%s\n", re_op_text[node->op]))
            if (text_ptr == text_end || text_ptr == state->final_newline)
                context->node = node->next_1;
            else
                context = reject_context(state, context);
            break;
        case RE_OP_GREEDY_REPEAT: /* Greedy limited repeat. */
        case RE_OP_LAZY_REPEAT: /* Lazy limited repeat. */
        {
            BOOL greedy;
            int rep_index;
            RE_ContextValue* info;
            BOOL consumed;
            BOOL try_body;
            RE_ContextNode* body_node;
            RE_CHAR* body_text_ptr;
            BOOL try_tail;
            RE_ContextNode* tail_node;
            RE_CHAR* tail_text_ptr;
            TRACE(("%s %d %d %d\n", re_op_text[node->op], node->values[0],
              node->values[1], node->values[2]))

            /* Is it a greedy or a lazy repeat? */
            greedy = node->op == RE_OP_GREEDY_REPEAT;

            /* Point to the current info for this repeat.
             *
             * We don't follow the body if the count has reached its maximum and
             * we don't follow the tail unless the count has reached its
             * minimum.
             *
             * Normally, if the body didn't consume any characters then it will
             * continue not consuming until the maximum count, at which point it
             * will follow the tail. We can take a shortcut by abandoning
             * further repeats of the body if it doesn't consume any characters.
             */
            rep_index = node->values[0];
            info = &context->values[rep_index];

            /* Test the 'body' and 'tail' for possible matches. */
            consumed = text_ptr != info->repeat.start;
            try_body = consumed && info->repeat.count < node->values[2] &&
              RE_TRY_MATCH(node->next_1, context, state, &body_node,
              &body_text_ptr);
            try_tail = (!consumed || info->repeat.count >= node->values[1]) &&
              RE_TRY_MATCH(node->next_2, context, state, &tail_node,
              &tail_text_ptr);
            if (try_body) {
                if (try_tail) {
                    RE_Context* body_context;
                    RE_ContextValue* body_info;
                    RE_Context* tail_context;
                    RE_ContextValue* tail_info;

                    if (greedy) {
                        /* Greedy repeat, so the body takes precedence. */
                        body_context = context;
                        body_info = info;
                        tail_context = copy_context_after(state, context);
                        tail_info = &tail_context->values[rep_index];
                    } else {
                        /* Lazy repeat, so the tail takes precedence. */
                        body_context = copy_context_after(state, context);
                        body_info = &body_context->values[rep_index];
                        tail_context = context;
                        tail_info = info;
                    }

                    /* Update the contexts for the body and tail.
                     *
                     * We also need to record where the text pointer is when we
                     * start on the body.
                     */
                    ++body_info->repeat.count;
                    body_info->repeat.start = body_context->text_ptr;
                    body_context->node = body_node;
                    body_context->text_ptr = body_text_ptr;
                    tail_info->repeat.count = 0;
                    tail_info->repeat.start = NULL;
                    tail_context->node = tail_node;
                    tail_context->text_ptr = tail_text_ptr;
                } else {
                    /* Update the context for the body.
                     *
                     * We also need to record where the text pointer is when we
                     * start on the body.
                     */
                    ++info->repeat.count;
                    info->repeat.start = text_ptr;
                    context->node = body_node;
                    context->text_ptr = body_text_ptr;
                }
            } else {
                if (try_tail) {
                    /* Update the contexts for the tail. */
                    info->repeat.count = 0;
                    info->repeat.start = NULL;
                    context->node = tail_node;
                    context->text_ptr = tail_text_ptr;
                } else
                    context = reject_context(state, context);
            }
            break;
        }
        case RE_OP_GROUP_EXISTS: /* Capture group exists. */
        {
            int g;
            RE_GroupInfo* info;
            RE_CHAR* begin_group;
            RE_CHAR* end_group;
            TRACE(("%s %d\n", re_op_text[node->op], node->values[0]))

            /* Group id. */
            g = node->values[0];

            info = &group_info_list[g - 1];
            g = info->value_offset;

            /* Check whether the captured text, if any, exists at this position
             * in the string.
             */
            begin_group = context->values[g].group.begin;
            end_group = context->values[g].group.end;
            if ((RE_CHAR*)context->slice_start <= begin_group && begin_group <=
              end_group && end_group <= (RE_CHAR*)context->slice_end)
                context->node = node->next_1;
            else
                context->node = node->next_2;
            break;
        }
        case RE_OP_LOOKAROUND: /* Lookaround. */
        {
            RE_Context* previous;
            RE_Context* next;
            RE_CHAR* slice_start;
            RE_CHAR* slice_end;
            int status;
            BOOL matched;
            TRACE(("%s\n", re_op_text[node->op]))

            /* The next_2 member of the node points to the subpattern which we
             * need to try.
             *
             * We need to change the context into a base context, which we'll do
             * by isolating it from the other contexts and then re-inserting it
             * afterwards.
             */

            /* Point to the subpattern. */
            context->node = node->next_2;

            /* Isolate the context. */
            previous = context->previous;
            next = context->next;
            context->previous = context;
            context->next = context;

            /* Save the old slice. */
            slice_start = context->slice_start;
            slice_end = context->slice_end;
            context->slice_start = text_start;
            context->slice_end = text_end;

            /* Try to match. */
            status = RE_MATCH_CONTEXT(state, context, FALSE, TRUE);

            /* Restore the old slice. */
            context->slice_start = slice_start;
            context->slice_end = slice_end;

            /* Re-insert the context. */
            context->previous = previous;
            context->next = next;

            if (status < 0) {
                free_all_contexts(state, base_context);
                return status;
            }

            matched = status == RE_ERROR_SUCCESS;
            if (matched == node->values[0])
                context->node = node->next_1;
            else
                context = reject_context(state, context);
            break;
        }
        case RE_OP_REF_GROUP: /* Reference to a capture group. */
        {
            int g;
            RE_CHAR* begin_group;
            RE_CHAR* end_group;
            TRACE(("%s %d\n", re_op_text[node->op], node->values[0]))

            /* Get the offset to the group value in the context. */
            g = node->values[0];
            g = group_info_list[g - 1].value_offset;

            /* Check whether the captured text, if any, exists at this position
             * in the string.
             */
            begin_group = context->values[g].group.begin;
            end_group = context->values[g].group.end;
            if ((RE_CHAR*)context->slice_start <= begin_group && begin_group <=
              end_group && end_group <= (RE_CHAR*)context->slice_end) {
                Py_ssize_t length;
                Py_ssize_t available;
                BOOL match;
                Py_ssize_t i;

                length = end_group - begin_group;
                available = (RE_CHAR*)context->slice_end - text_ptr;

                match = length <= available;
                for (i = 0; match && i < length; i++)
                    match = text_ptr[i] == begin_group[i];
                text_ptr += length;

                if (match) {
                    context->text_ptr = text_ptr;
                    context->node = node->next_1;
                } else
                    context = reject_context(state, context);
            } else
                context = reject_context(state, context);
            break;
        }
        case RE_OP_REF_GROUP_IGNORE: /* Reference to a capture group, ignoring case. */
        {
            int g;
            RE_CHAR* begin_group;
            RE_CHAR* end_group;
            TRACE(("%s %d\n", re_op_text[node->op], node->values[0]))

            /* Get the offset to the group value in the context. */
            g = node->values[0];
            g = group_info_list[g - 1].value_offset;

            /* Check whether the captured text, if any, exists at this position
             * in the string.
             */
            begin_group = context->values[g].group.begin;
            end_group = context->values[g].group.end;
            if ((RE_CHAR*)context->slice_start <= begin_group && begin_group <=
              end_group && end_group <= (RE_CHAR*)context->slice_end) {
                Py_ssize_t length;
                Py_ssize_t available;
                BOOL match;
                Py_ssize_t i;

                length = end_group - begin_group;
                available = (RE_CHAR*)context->slice_end - text_ptr;

                match = length <= available;
                for (i = 0; match && i < length; i++)
                    match = same_char_ignore(encoding, text_ptr[i],
                      begin_group[i]);
                text_ptr += length;

                if (match) {
                    context->text_ptr = text_ptr;
                    context->node = node->next_1;
                } else
                    context = reject_context(state, context);
            } else
                context = reject_context(state, context);
            break;
        }
        case RE_OP_REF_GROUP_IGNORE_REV: /* Reference to a capture group, ignoring case. */
        {
            int g;
            RE_CHAR* begin_group;
            RE_CHAR* end_group;
            TRACE(("%s %d\n", re_op_text[node->op], node->values[0]))

            /* Get the offset to the group value in the context. */
            g = node->values[0];
            g = group_info_list[g - 1].value_offset;

            /* Check whether the captured text, if any, exists at this position
             * in the string.
             */
            begin_group = context->values[g].group.begin;
            end_group = context->values[g].group.end;
            if ((RE_CHAR*)context->slice_start <= begin_group && begin_group <=
              end_group && end_group <= (RE_CHAR*)context->slice_end) {
                Py_ssize_t length;
                Py_ssize_t available;
                BOOL match;
                Py_ssize_t i;

                length = end_group - begin_group;
                available = text_ptr - (RE_CHAR*)context->slice_start;

                match = length <= available;
                text_ptr -= length;
                for (i = 0; match && i < length; i++)
                    match = same_char_ignore(encoding, text_ptr[i],
                      begin_group[i]);

                if (match) {
                    context->text_ptr = text_ptr;
                    context->node = node->next_1;
                } else
                    context = reject_context(state, context);
            } else
                context = reject_context(state, context);
            break;
        }
        case RE_OP_REF_GROUP_REV: /* Reference to a capture group. */
        {
            int g;
            RE_CHAR* begin_group;
            RE_CHAR* end_group;
            TRACE(("%s %d\n", re_op_text[node->op], node->values[0]))

            /* Get the offset to the group value in the context. */
            g = node->values[0];
            g = group_info_list[g - 1].value_offset;

            /* Check whether the captured text, if any, exists at this position
             * in the string.
             */
            begin_group = context->values[g].group.begin;
            end_group = context->values[g].group.end;
            if ((RE_CHAR*)context->slice_start <= begin_group && begin_group <=
              end_group && end_group <= (RE_CHAR*)context->slice_end) {
                Py_ssize_t length;
                Py_ssize_t available;
                BOOL match;
                Py_ssize_t i;

                length = end_group - begin_group;
                available = text_ptr - (RE_CHAR*)context->slice_start;

                match = length <= available;
                text_ptr -= length;
                for (i = 0; match && i < length; i++)
                    match = text_ptr[i] == begin_group[i];

                if (match) {
                    context->text_ptr = text_ptr;
                    context->node = node->next_1;
                } else
                    context = reject_context(state, context);
            } else
                context = reject_context(state, context);
            break;
        }
        case RE_OP_SET: /* Character set. */
        {
            TRACE(("%s %d\n", re_op_text[node->op], node->values[0]))
            if (text_ptr < (RE_CHAR*)context->slice_end && in_set(state,
              node->values, text_ptr[0]) == node->values[0]) {
                context->node = node->next_1;
                ++(RE_CHAR*)context->text_ptr;
            } else
                context = reject_context(state, context);
            break;
        }
        case RE_OP_SET_IGNORE: /* Character set, ignoring case. */
        {
            TRACE(("%s %d\n", re_op_text[node->op], node->values[0]))
            if (text_ptr < (RE_CHAR*)context->slice_end && in_set_ignore(state,
              node->values, text_ptr[0]) == node->values[0]) {
                context->node = node->next_1;
                ++(RE_CHAR*)context->text_ptr;
            } else
                context = reject_context(state, context);
            break;
        }
        case RE_OP_SET_IGNORE_REV: /* Character set, ignoring case. */
        {
            TRACE(("%s %d\n", re_op_text[node->op], node->values[0]))
            if (text_ptr > (RE_CHAR*)context->slice_start &&
              in_set_ignore(state, node->values, text_ptr[-1]) ==
              node->values[0]) {
                context->node = node->next_1;
                --(RE_CHAR*)context->text_ptr;
            } else
                context = reject_context(state, context);
            break;
        }
        case RE_OP_SET_REV: /* Character set. */
        {
            TRACE(("%s %d\n", re_op_text[node->op], node->values[0]))
            if (text_ptr > (RE_CHAR*)context->slice_start && in_set(state,
              node->values, text_ptr[-1]) == node->values[0]) {
                context->node = node->next_1;
                --(RE_CHAR*)context->text_ptr;
            } else
                context = reject_context(state, context);
            break;
        }
        case RE_OP_START_OF_LINE: /* At the start of a line. */
            TRACE(("%s\n", re_op_text[node->op]))
            if (text_ptr == text_start || text_ptr[-1] == '\n')
                context->node = node->next_1;
            else
                context = reject_context(state, context);
            break;
        case RE_OP_START_OF_STRING: /* At the start of the string. */
            TRACE(("%s\n", re_op_text[node->op]))
            if (text_ptr == text_start)
                context->node = node->next_1;
            else
                context = reject_context(state, context);
            break;
        case RE_OP_SUCCESS: /* Success. */
        {
            BOOL consumed;

            /* This context has succeeded, but we aren't allowed to return a
             * series of zero-width match because that would result in an
             * infinite loop, never advancing any further through the string.
             */
            consumed = text_ptr != context->slice_start;
            if (consumed || context->allow_zero_width) {
                /* This context has succeeded and is acceptable, so we can
                 * discard any later ones.
                 */
                while (context->next != base_context)
                    delete_context(state, context->next);

                context = base_context;

                /* Now that we have a possible match we can stop creating
                 * injecting new contexts if we're searching rather than just
                 * matching.
                 */
                search = FALSE;
            } else
                context = reject_context(state, context);
            break;
        }
        case RE_OP_UNLIMITED_GREEDY_REPEAT: /* Greedy unlimited repeat. */
        case RE_OP_UNLIMITED_LAZY_REPEAT: /* Lazy unlimited repeat. */
        {
            BOOL greedy;
            int rep_index;
            RE_ContextValue* info;
            BOOL consumed;
            BOOL try_body;
            RE_ContextNode* body_node;
            RE_CHAR* body_text_ptr;
            BOOL try_tail;
            RE_ContextNode* tail_node;
            RE_CHAR* tail_text_ptr;
            TRACE(("%s %d\n", re_op_text[node->op], node->values[0]))

            /* Is it a greedy or a lazy repeat? */
            greedy = node->op == RE_OP_UNLIMITED_GREEDY_REPEAT;

            /* Point to the current info for this repeat.
             *
             * We don't follow the body if the count has reached its maximum and
             * we don't follow the tail unless the count has reached its
             * minimum.
             *
             * Normally, if the body didn't consume any characters then it will
             * continue not consuming until the maximum count, at which point it
             * will follow the tail. We can take a shortcut by abandoning
             * further repeats of the body if it doesn't consume any characters.
             */
            rep_index = node->values[0];
            info = &context->values[rep_index];

            /* Test the 'body' and 'tail' for possible matches. */
            consumed = text_ptr != info->repeat.start;
            try_body = consumed && RE_TRY_MATCH(node->next_1, context, state,
              &body_node, &body_text_ptr);
            try_tail = RE_TRY_MATCH(node->next_2, context, state, &tail_node,
              &tail_text_ptr);
            if (try_body) {
                if (try_tail) {
                    RE_Context* body_context;
                    RE_ContextValue* body_info;
                    RE_Context* tail_context;
                    RE_ContextValue* tail_info;

                    if (greedy) {
                        /* Greedy repeat, so the body takes precedence. */
                        body_context = context;
                        body_info = info;
                        tail_context = copy_context_after(state, context);
                        tail_info = &tail_context->values[rep_index];
                    } else {
                        /* Lazy repeat, so the tail takes precedence. */
                        body_context = copy_context_after(state, context);
                        body_info = &body_context->values[rep_index];
                        tail_context = context;
                        tail_info = info;
                    }

                    /* Update the contexts for the body and tail.
                     *
                     * We also need to record where the text pointer is when we
                     * start on the body.
                     */
                    body_info->repeat.start = body_context->text_ptr;
                    body_context->node = body_node;
                    body_context->text_ptr = body_text_ptr;
                    tail_info->repeat.start = NULL;
                    tail_context->node = tail_node;
                    tail_context->text_ptr = tail_text_ptr;
                } else {
                    /* Update the context for the body.
                     *
                     * We also need to record where the text pointer is when we
                     * start on the body.
                     */
                    info->repeat.start = text_ptr;
                    context->node = body_node;
                    context->text_ptr = body_text_ptr;
                }
            } else {
                if (try_tail) {
                    /* Update the context for the tail. */
                    info->repeat.start = NULL;
                    context->node = tail_node;
                    context->text_ptr = tail_text_ptr;
                } else
                    context = reject_context(state, context);
            }
            break;
        }
        default: /* Illegal opcode! */
            TRACE(("UNKNOWN OP %d\n", node->op))
            return RE_ERROR_ILLEGAL;
        }

        ++iterations;
        if (depth_first) {
            /* Working depth-first. */

            /* Should we continue working depth-first? */
            depth_first = iterations - start_iterations < RE_MAX_ITERATIONS;
        } else if (context != base_context) {
            /* Working breadth-first. */
            RE_Context* next;

            /* Should we abort the matching? */
            if ((iterations & 0xFFFF) == 0) {
                if (PyErr_CheckSignals())
                    return RE_ERROR_INTERRUPTED;

                iterations = 0;
            }

            /* Advance to the next context if we're further ahead with this one.
             */
            next = context->next;
            if (context->text_ptr > next->text_ptr) {
                if (can_merge_contexts(state, context))
                    /* We've already seen a similar context, so discard this one
                     * to avoid duplicating work.
                     */
                    delete_context(state, context);
                else
                    /* Save this context for later. */
                    add_context(context->node, context);

                context = next;
            }
        }

        /* Have we done all the contexts? */
        if (context == base_context) {
#if defined(VERBOSE)
            int count;

            context = base_context->next;
            count = 0;
            while (context != base_context) {
                context = context->next;
                ++count;
            }
            printf("%d contexts\n", count);
#endif

            /* Is the first context successful? */
            context = base_context->next;
            if (context->node->op == RE_OP_SUCCESS) {
                /* Store the results in the base context.
                 *
                 * Some things shouldn't be stored if we're doing a lookahead.
                 */
                int i;

                if (!lookahead) {
                    base_context->slice_start = context->slice_start;
                    base_context->text_ptr = context->text_ptr;
                    base_context->allow_zero_width = context->allow_zero_width;
                }
                base_context->lastindex = context->lastindex;
                base_context->lastgroup = context->lastgroup;

                /* Store the capture groups. */
                for (i = 0; i < state->pattern->group_count; i ++) {
                    int g;

                    g = group_info_list[i].value_offset;
                    base_context->values[g].group.begin =
                      context->values[g].group.begin;
                    base_context->values[g].group.end =
                      context->values[g].group.end;
                }

                /* Delete all but the base context. */
                delete_context(state, context);

                return RE_ERROR_SUCCESS;
            }

            /* If we're searching then we need to inject another context at the
             * next start position.
             *
             * When working depth-first this will happen only when all the
             * existing contexts have been discarded.
             */
            if (search) {
                base_context->allow_zero_width = TRUE;
                start_iterations = iterations;

                for (;;) {
                    RE_ContextNode* next_node;
                    RE_CHAR* next_text_ptr;

                    /* Stop searching when we've reached the end of the slice.
                     */
                    search = base_context->slice_start <
                      base_context->slice_end;
                    if (!search)
                        break;

                    ++(RE_CHAR*)base_context->slice_start;
                    base_context->text_ptr = base_context->slice_start;

                    /* There's no point injecting the new context if it's the
                     * same as one we've already seen or it won't match here.
                     */
                    if ((context == base_context || !same_context(state,
                      base_context, context)) &&
                      RE_TRY_MATCH(base_context->node, base_context, state,
                      &next_node, &next_text_ptr)) {
                        /* Inject the new context at the end of the list. */
                        copy_context_before(state, base_context);
                        context = base_context->previous;
                        context->node = next_node;
                        context->text_ptr = next_text_ptr;
                        context = base_context->next;
                        break;
                    }
                }
            }

            /* If we still don't have any contexts, then the search or match has
             * failed.
             */
            if (context == base_context)
                break;

            /* Reset the context nodes for the next pass.
             * This isn't necessary in depth-first mode.
             */
            if (!depth_first) {
                int i;

                for (i = 0; i < node_count; i++)
                    node_list[i].context_count = 0;
            }
        }
    }

    return RE_ERROR_FAILURE;
}

Py_LOCAL(int) RE_MATCH(RE_State* state, BOOL search) {
    RE_Context* base_context;
    int status;
    TRACE(("<<RE_MATCH>>\n"))

    /* Is there a slice to search? */
    if (state->slice_start > state->slice_end)
        return RE_ERROR_FAILURE;

    /* Point to the final newline if it's at the end of the string. */
    state->final_newline = state->text_end > state->text_start &&
      ((RE_CHAR*)state->text_end)[-1] == '\n' ? (RE_CHAR*)state->text_end - 1 :
      NULL;

    /* Create the base context which is the template for creating contexts. */
    base_context = create_base_context(state);
    if (!base_context)
        return RE_ERROR_MEMORY;

    /* Perform the match. */
    status = RE_MATCH_CONTEXT(state, base_context, search, FALSE);
    if (status == RE_ERROR_SUCCESS) {
        int i;

        /* Store the results. */
        state->slice_start = base_context->slice_start;
        state->text_ptr = base_context->text_ptr;
        state->allow_zero_width = base_context->allow_zero_width;
        state->lastindex = base_context->lastindex;
        state->lastgroup = base_context->lastgroup;

        /* Store the capture groups. */
        for (i = 0; i < state->pattern->group_count; i ++) {
            int m;
            int g;

            m = i * 2;
            g = state->pattern->group_info_list[i].value_offset;
            state->marks[m] = base_context->values[g].group.begin;
            state->marks[m + 1] = base_context->values[g].group.end;
            TRACE(("group %d at %d from %d to %d\n", i + 1, g, text_offset(state,
              state->marks[m]), text_offset(state, state->marks[m + 1])))
        }
    }

    free_all_contexts(state, base_context);

    return status;
}

#undef RE_CHAR
#undef RE_MATCH_CONTEXT
#undef RE_MATCH
#undef RE_LITERAL_TEMPLATE
#undef RE_AT_BOUNDARY
#undef RE_TRY_MATCH

#if !defined(RE_RECURSIVE)
/* Parsed on first (non-recursive) pass only.
 *
 * This causes the recursive (second) pass.
 */

#define RE_RECURSIVE

/* Set up for Unicode. */
#define RE_CHAR Py_UNICODE
#define RE_MATCH_CONTEXT umatch_context
#define RE_MATCH umatch
#define RE_LITERAL_TEMPLATE uliteral_template
#define RE_AT_BOUNDARY uat_boundary
#define RE_TRY_MATCH utry_match

#include "_regex.c"

/* Undefine so that the remainder of the code is skipped on the first pass (it
 * will have been done on the recursive pass).
 */
#undef RE_RECURSIVE
#endif

#if defined(RE_RECURSIVE)
/* Parsed on recursive (second) pass only. */

/* Gets a string from a Python object. */
Py_LOCAL(BOOL) getstring(PyObject* string, void** characters, Py_ssize_t*
  length, int* charsize) {
    /* Given a Python object, return a data pointer, a length (in characters),
     * and a character size.  Return FALSE if the object is not a string (or not
     * compatible).
     */
    PyBufferProcs* buffer;
    Py_ssize_t byte_size;

    if (PyUnicode_Check(string)) {
        /* Unicode strings doesn't always support the buffer interface. */
        *characters = (void*)PyUnicode_AS_DATA(string);
        byte_size = PyUnicode_GET_DATA_SIZE(string);
        *length = PyUnicode_GET_SIZE(string);
        *charsize = sizeof(Py_UNICODE);
    } else {
        /* Get pointer to string buffer. */
        buffer = string->ob_type->tp_as_buffer;
        if (!buffer || !buffer->bf_getreadbuffer || !buffer->bf_getsegcount ||
          buffer->bf_getsegcount(string, NULL) != 1) {
            PyErr_SetString(PyExc_TypeError, "expected string or buffer");
            return FALSE;
        }

        /* Determine buffer size. */
        byte_size = buffer->bf_getreadbuffer(string, 0, characters);
        if (byte_size < 0) {
            PyErr_SetString(PyExc_TypeError, "buffer has negative size");
            return FALSE;
        }

        /* Determine character size. */
        *length = PyObject_Size(string);
        if (PyString_Check(string) || byte_size == *length)
            *charsize = 1;
        else if (byte_size == (Py_ssize_t)(*length * sizeof(Py_UNICODE)))
            *charsize = sizeof(Py_UNICODE);
        else {
            PyErr_SetString(PyExc_TypeError, "buffer size mismatch");
            return FALSE;
        }
    }

    return TRUE;
}

/* Initialises a state object. */
Py_LOCAL(BOOL) state_init(RE_State* state, PatternObject* pattern, PyObject*
  string, Py_ssize_t start, Py_ssize_t end) {
    Py_ssize_t node_list_size;
    void* characters;
    Py_ssize_t length;
    int charsize;
    int i;

    memset(state, 0, sizeof(RE_State));

    /* Get the string to search or match. */
    if (!getstring(string, &characters, &length, &charsize))
        goto error;

    /* The marks for the capture groups. */
    state->marks = (void**)PyMem_MALLOC(pattern->group_count * 2 *
      sizeof(*state->marks));
    if (!state->marks)
        goto error;

    /* The context list. */
    state->context_capacity = 16;
    state->context_list = (RE_Context**)PyMem_MALLOC(state->context_capacity *
      sizeof(*state->context_list));
    if (!state->context_list)
        goto error;

    /* The context nodes. */
    node_list_size = (Py_ssize_t)(pattern->node_count * sizeof(*state->node_list));
    state->node_list = (RE_ContextNode*)PyMem_MALLOC(node_list_size);
    if (!state->node_list)
        goto error;
    memset(state->node_list, 0, node_list_size);

    for (i = 0; i < pattern->node_count; i++) {
        RE_PatternNode* pnode;
        RE_ContextNode* cnode;

        pnode = pattern->node_list[i];
        cnode = &state->node_list[pnode->index];
        cnode->op = pnode->op;
        cnode->values = pnode->values;
        cnode->context_capacity = 16;
        cnode->context_count = 0;
        cnode->context_list = (RE_Context**)PyMem_MALLOC(cnode->context_capacity
          * sizeof(*cnode->context_list));
        if (!cnode->context_list)
            goto error;
        if (pnode->next_1)
            cnode->next_1 = &state->node_list[pnode->next_1->index];
        if (pnode->next_2)
            cnode->next_2 = &state->node_list[pnode->next_2->index];
    }

    /* Adjust boundaries. */
    if (start < 0)
        start = 0;
    else if (start > length)
        start = length;

    if (end < 0)
        end = 0;
    else if (end > length)
        end = length;

    state->charsize = charsize;

    /* Initialise the character encoding. If the character size is larger than a
     * byte then we'll assume it's Unicode.
     */
    if (charsize > 1)
        state->encoding = &unicode_encoding;
    else if (pattern->flags & RE_FLAG_LOCALE)
        state->encoding = &locale_encoding;
    else
        state->encoding = &ascii_encoding;

    /* The state object contsins a reference to the string and also a pointer to
     * its contents.
     */
    state->text_start = characters;
    state->text_end = (void*)((char*)characters + length * state->charsize);

    state->slice_start = (void*)((char*)characters + start * state->charsize);
    state->slice_end = (void*)((char*)characters + end * state->charsize);

    state->zero_width = (pattern->flags & RE_FLAG_ZEROWIDTH) != 0;
    state->allow_zero_width = TRUE;

    state->pattern = pattern;
    state->string = string;
    Py_INCREF(state->pattern);
    Py_INCREF(state->string);

    return TRUE;

error:
    set_error(RE_ERROR_MEMORY, NULL);

    PyMem_FREE(state->marks);

    for (i = 0; i < pattern->node_count; i++)
        PyMem_FREE(state->node_list[i].context_list);

    PyMem_FREE(state->node_list);

    return FALSE;
}

/* Finalises a state objects, discarding its contents. */
Py_LOCAL(void) state_fini(RE_State* state) {
    int i;

    Py_DECREF(state->pattern);
    Py_DECREF(state->string);
    PyMem_FREE(state->marks);

    for (i = 0; i < state->context_count; i++)
        PyMem_FREE(state->context_list[i]);

    for (i = 0; i < state->pattern->node_count; i++)
        PyMem_FREE(state->node_list[i].context_list);

    PyMem_FREE(state->node_list);
}

/* Deallocates a MatchObject. */
static void match_dealloc(MatchObject* self) {
    Py_XDECREF(self->string);
    Py_DECREF(self->pattern);
    PyMem_FREE(self->marks);
    Py_XDECREF(self->regs);
    PyObject_DEL(self);
}

/* Gets a MatchObject's group by integer index. */
Py_LOCAL(PyObject*) match_get_group_by_index(MatchObject* self, Py_ssize_t
  index, PyObject* def) {
    Py_ssize_t m;

    if (index < 0 || index > self->group_count) {
        /* Raise IndexError if we were given a bad group number. */
        set_error(RE_ERROR_GROUP, NULL);
        return NULL;
    }

    if (index == 0)
        return PySequence_GetSlice(self->string, self->pos, self->endpos);

    m = index * 2 - 2;

    if (self->string == Py_None || self->marks[m] < 0 || self->marks[m + 1] < 0)
      {
        /* Return default value if the string or group is undefined. */
        Py_INCREF(def);
        return def;
    }

    return PySequence_GetSlice(self->string, self->marks[m], self->marks[m +
      1]);
}

/* Gets a MatchObject's group index.
 *
 * The supplied index can be an integer or a string (group name) object.
 */
Py_LOCAL(Py_ssize_t) match_get_group_index(MatchObject* self, PyObject* index,
  BOOL allow_neg) {
    Py_ssize_t group = -1;

    /* Is the index an integer? */
    if (PyInt_Check(index) || PyLong_Check(index)) {
        group = PyInt_AsSsize_t(index);
        /* Adjust negative indices where valid and allowed. */
        if (allow_neg && -self->group_count <= group && group <= -1)
            group += self->group_count + 1;
    } else {
        /* The index might be a group name. */
        if (self->pattern->groupindex) {
            /* Look up the name. */
            index = PyObject_GetItem(self->pattern->groupindex, index);
            if (index) {
                /* check that we have an integer. */
                if (PyInt_Check(index) || PyLong_Check(index))
                    group = PyInt_AsSsize_t(index);

                Py_DECREF(index);
            } else
                PyErr_Clear();
        }
    }

    return group;
}

/* Gets a MatchObject's group by object index. */
Py_LOCAL(PyObject*) match_get_group(MatchObject* self, PyObject* index,
  PyObject* def, BOOL allow_neg) {
    /* Check that the index is an integer or a string. */
    if (!PyInt_Check(index) && !PyLong_Check(index) && !PyString_Check(index) &&
      !PyUnicode_Check(index)) {
          set_error(RE_ERROR_INDEX, index);
        return NULL;
    }

    return match_get_group_by_index(self, match_get_group_index(self, index,
      allow_neg), def);
}

/* MatchObject's 'group' method. */
static PyObject* match_group(MatchObject* self, PyObject* args) {
    Py_ssize_t size;
    PyObject* result;
    Py_ssize_t i;

    size = PyTuple_GET_SIZE(args);

    switch (size) {
    case 0:
        /* group() */
        result = match_get_group_by_index(self, 0, Py_None);
        break;
    case 1:
        /* group(x) */
        result = match_get_group(self, PyTuple_GET_ITEM(args, 0), Py_None,
          FALSE);
        break;
    default:
        /* group(x, y, z, ...) */
        /* Fetch multiple items. */
        result = PyTuple_New(size);
        if (!result)
            return NULL;
        for (i = 0; i < size; i++) {
            PyObject* item = match_get_group(self, PyTuple_GET_ITEM(args, i),
              Py_None, FALSE);
            if (!item) {
                Py_DECREF(result);
                return NULL;
            }
            PyTuple_SET_ITEM(result, i, item);
        }
        break;
    }
    return result;
}

/* MatchObject's 'start' method. */
static PyObject* match_start(MatchObject* self, PyObject* args) {
    Py_ssize_t index;

    PyObject* index_ = Py_False; /* Default index is 0. */
    if (!PyArg_UnpackTuple(args, "start", 0, 1, &index_))
        return NULL;

    index = match_get_group_index(self, index_, FALSE);
    if (index < 0 || index > self->group_count) {
        set_error(RE_ERROR_GROUP, NULL);
        return NULL;
    }

    if (index == 0)
        return Py_BuildValue("i", self->pos);

    /* A mark is -1 if the group is undefined. */
    return Py_BuildValue("i", self->marks[index * 2 - 2]);
}

/* MatchObject's 'end' method. */
static PyObject* match_end(MatchObject* self, PyObject* args) {
    Py_ssize_t index;

    PyObject* index_ = Py_False; /* Default index is 0. */
    if (!PyArg_UnpackTuple(args, "end", 0, 1, &index_))
        return NULL;

    index = match_get_group_index(self, index_, FALSE);

    if (index < 0 || index > self->group_count) {
        set_error(RE_ERROR_GROUP, NULL);
        return NULL;
    }

    if (index == 0)
        return Py_BuildValue("i", self->endpos);

    /* A mark is -1 if the group is undefined. */
    return Py_BuildValue("i", self->marks[index * 2 - 1]);
}

/* Creates an integer pair (2-tuple). */
Py_LOCAL(PyObject*) _pair(Py_ssize_t i1, Py_ssize_t i2) {
    PyObject* pair;
    PyObject* item;

    pair = PyTuple_New(2);
    if (!pair)
        return NULL;

    item = PyInt_FromSsize_t(i1);
    if (!item)
        goto error;
    PyTuple_SET_ITEM(pair, 0, item);

    item = PyInt_FromSsize_t(i2);
    if (!item)
        goto error;
    PyTuple_SET_ITEM(pair, 1, item);

    return pair;

error:
    Py_DECREF(pair);
    return NULL;
}

/* MatchObject's 'span' method. */
static PyObject* match_span(MatchObject* self, PyObject* args) {
    Py_ssize_t index;

    PyObject* index_ = Py_False; /* Default index is 0. */
    if (!PyArg_UnpackTuple(args, "span", 0, 1, &index_))
        return NULL;

    index = match_get_group_index(self, index_, FALSE);
    if (index < 0 || index > self->group_count) {
        set_error(RE_ERROR_GROUP, NULL);
        return NULL;
    }

    if (index == 0)
        return _pair(self->pos, self->endpos);

    /* Marks are -1 if group is undefined. */
    return _pair(self->marks[index * 2 - 2], self->marks[index * 2 - 1]);
}

/* MatchObject's 'groups' method. */
static PyObject* match_groups(MatchObject* self, PyObject* args, PyObject* kw) {
    PyObject* result;
    Py_ssize_t index;

    PyObject* def = Py_None;
    static char* kwlist[] = { "default", NULL };
    if (!PyArg_ParseTupleAndKeywords(args, kw, "|O:groups", kwlist, &def))
        return NULL;

    result = PyTuple_New(self->group_count);
    if (!result)
        return NULL;

    /* Group 0 is the entire matched portion of the string. */
    for (index = 0; index < self->group_count; index++) {
        PyObject* item;
        item = match_get_group_by_index(self, index + 1, def);
        if (!item) {
            Py_DECREF(result);
            return NULL;
        }
        PyTuple_SET_ITEM(result, index, item);
    }

    return result;
}

/* MatchObject's 'groupdict' method. */
static PyObject* match_groupdict(MatchObject* self, PyObject* args, PyObject*
  kw) {
    PyObject* result;
    PyObject* keys;
    Py_ssize_t index;

    PyObject* def = Py_None;
    static char* kwlist[] = { "default", NULL };
    if (!PyArg_ParseTupleAndKeywords(args, kw, "|O:groupdict", kwlist, &def))
        return NULL;

    result = PyDict_New();
    if (!result || !self->pattern->groupindex)
        return result;

    keys = PyMapping_Keys(self->pattern->groupindex);
    if (!keys)
        goto failed;

    for (index = 0; index < PyList_GET_SIZE(keys); index++) {
        int status;
        PyObject* key;
        PyObject* value;
        key = PyList_GET_ITEM(keys, index);
        if (!key)
            goto failed;
        value = match_get_group(self, key, def, FALSE);
        if (!value) {
            Py_DECREF(key);
            goto failed;
        }
        status = PyDict_SetItem(result, key, value);
        Py_DECREF(value);
        if (status < 0)
            goto failed;
    }

    Py_DECREF(keys);

    return result;

failed:
    Py_XDECREF(keys);
    Py_DECREF(result);
    return NULL;
}

/* Calls a function in the re module. */
Py_LOCAL(PyObject*) call(char* module, char* function, PyObject* args) {
    PyObject* name;
    PyObject* mod;
    PyObject* func;
    PyObject* result;

    if (!args)
        return NULL;
    name = PyString_FromString(module);
    if (!name)
        return NULL;
    mod = PyImport_Import(name);
    Py_DECREF(name);
    if (!mod)
        return NULL;
    func = PyObject_GetAttrString(mod, function);
    Py_DECREF(mod);
    if (!func)
        return NULL;
    result = PyObject_CallObject(func, args);
    Py_DECREF(func);
    Py_DECREF(args);
    return result;
}

/* Gets a replacement item from the replacement list.
 *
 * The replacement item could be a string literal or a group.
 */
PyObject* get_match_replacement(MatchObject* self, PyObject* item, PyObject*
  string, int group_count) {
    Py_ssize_t group;

    if (PyString_Check(item) || PyUnicode_Check(item)) {
        /* It's a literal, which can be added directly to the list. */
        Py_INCREF(item);
        return item;
    }

    /* Is it a group reference? */
    if (PyInt_Check(item) || PyLong_Check(item))
        group = PyInt_AsSsize_t(item);
    else {
        /* Not a group either! */
        set_error(RE_ERROR_REPLACEMENT, NULL);
        return NULL;
    }

    if (group == 0) {
        /* The entire matched portion of the string. */
        return PySequence_GetSlice(string, self->pos, self->endpos);
    } else if (group >= 1 && group <= group_count) {
        /* A group. If it didn't match then return None instead. */
        Py_ssize_t m = group * 2 - 2;
        if (self->marks[m] >= 0)
            return PySequence_GetSlice(string, self->marks[m], self->marks[m +
              1]);
        else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    } else {
        /* No such group. */
        set_error(RE_ERROR_GROUP, NULL);
        return NULL;
    }
}

/* Joins together a list of strings. */
Py_LOCAL(PyObject*) join_list(PyObject* list, PyObject* string) {
    /* Join list elements. */
    PyObject* joiner;
    PyObject* function;
    PyObject* args;
    PyObject* result;

    joiner = PySequence_GetSlice(string, 0, 0);
    if (!joiner)
        return NULL;

    if (PyList_GET_SIZE(list) == 0) {
        Py_DECREF(list);
        return joiner;
    }

    function = PyObject_GetAttrString(joiner, "join");
    if (!function) {
        Py_DECREF(joiner);
        return NULL;
    }
    args = PyTuple_New(1);
    if (!args) {
        Py_DECREF(function);
        Py_DECREF(joiner);
        return NULL;
    }
    PyTuple_SET_ITEM(args, 0, list);
    result = PyObject_CallObject(function, args);
    Py_DECREF(args); /* Also removes list. */
    Py_DECREF(function);
    Py_DECREF(joiner);

    return result;
}

/* MatchObject's 'expand' method. */
static PyObject* match_expand(MatchObject* self, PyObject* str_template) {
    PyObject* replacement;
    PyObject* list;
    Py_ssize_t size;
    Py_ssize_t i;

    /* Hand the template to the template compiler. */
    replacement = call(RE_MODULE, "_compile_replacement", PyTuple_Pack(2,
      self->pattern, str_template));
    if (!replacement)
        return NULL;

    list = PyList_New(0);
    if (!list) {
        Py_DECREF(replacement);
        return NULL;
    }

    /* Add each part of the template to the list. */
    size = PyList_GET_SIZE(replacement);
    for (i = 0; i < size; i++) {
        PyObject* item;
        PyObject* str_item;

        item = PyList_GET_ITEM(replacement, i);
        str_item = get_match_replacement(self, item, self->string,
          self->group_count);
        if (!str_item)
            goto error;

        /* Add to the list. */
        if (str_item == Py_None)
            Py_DECREF(str_item);
        else {
            int status;

            status = PyList_Append(list, str_item);
            Py_DECREF(str_item);
            if (status < 0) {
                set_error(status, NULL);
                goto error;
            }
        }
    }

    Py_DECREF(replacement);

    /* Convert the list to a single string (also removes the list). */
    return join_list(list, self->string);

error:
    Py_DECREF(list);
    Py_DECREF(replacement);
    return NULL;
}

/* MatchObject's 'copy' method. */
static PyObject* match_copy(MatchObject* self, PyObject *unused) {
    PyErr_SetString(PyExc_TypeError, "cannot copy this MatchObject");
    return NULL;
}

/* MatchObject's 'deepcopy' method. */
static PyObject* match_deepcopy(MatchObject* self, PyObject* memo) {
    PyErr_SetString(PyExc_TypeError, "cannot deepcopy this MatchObject");
    return NULL;
}

/* MatchObject's 'regs' method. */
static PyObject* match_regs(MatchObject* self) {
    PyObject* regs;
    PyObject* item;
    Py_ssize_t index;

    regs = PyTuple_New(self->group_count + 1);
    if (!regs)
        return NULL;

    item = _pair(self->pos, self->endpos);
    if (!item) {
        Py_DECREF(regs);
        return NULL;
    }
    PyTuple_SET_ITEM(regs, 0, item);

    for (index = 0; index < self->group_count; index++) {
        item = _pair(self->marks[index * 2], self->marks[index * 2 + 1]);
        if (!item) {
            Py_DECREF(regs);
            return NULL;
        }
        PyTuple_SET_ITEM(regs, index + 1, item);
    }

    Py_INCREF(regs);
    self->regs = regs;

    return regs;
}

/* Gets the value of an index. */
Py_LOCAL(BOOL) get_index(PyObject* item, Py_ssize_t* value, Py_ssize_t def) {
    if (item == Py_None)
        *value = def;
    else if (PyInt_Check(item) || PyLong_Check(item))
        *value = PyInt_AsSsize_t(item);
    else
        return FALSE;

    return TRUE;
}

/* MatchObject's slice method. */
Py_LOCAL(PyObject*) match_get_group_slice(MatchObject* self, PySliceObject*
  slice) {
    Py_ssize_t start;
    Py_ssize_t stop;
    Py_ssize_t step;
    Py_ssize_t slice_length;

    if (PySlice_GetIndicesEx(slice, self->group_count + 1, &start, &stop, &step,
      &slice_length) < 0) {
		return NULL;
	}

	if (slice_length <= 0)
		return PyTuple_New(0);
	else {
        PyObject* result;
        Py_ssize_t cur;
        Py_ssize_t i;

        result = PyTuple_New(slice_length);
		if (!result)
            return NULL;

		cur = start;
        for (i = 0; i < slice_length; i++) {
            PyTuple_SetItem(result, i, match_get_group_by_index(self, cur,
              Py_None));
            cur += step;
        }

		return result;
	}
}

/* MatchObject's length method. */
static Py_ssize_t match_length(MatchObject* self) {
    return self->group_count + 1;
}

/* MatchObject's subscript method. */
static PyObject* match_subscript(MatchObject* self, PyObject* item) {
    if (PySlice_Check(item))
        return match_get_group_slice(self, (PySliceObject*)item);

    return match_get_group(self, item, Py_None, TRUE);
}

/* MatchObject's methods. */
static PyMethodDef match_methods[] = {
    {"group", (PyCFunction)match_group, METH_VARARGS},
    {"start", (PyCFunction)match_start, METH_VARARGS},
    {"end", (PyCFunction)match_end, METH_VARARGS},
    {"span", (PyCFunction)match_span, METH_VARARGS},
    {"groups", (PyCFunction)match_groups, METH_VARARGS|METH_KEYWORDS},
    {"groupdict", (PyCFunction)match_groupdict, METH_VARARGS|METH_KEYWORDS},
    {"expand", (PyCFunction)match_expand, METH_O},
    {"__copy__", (PyCFunction)match_copy, METH_NOARGS},
    {"__deepcopy__", (PyCFunction)match_deepcopy, METH_O},
    {"__getitem__", (PyCFunction)match_subscript, METH_O|METH_COEXIST},
    {NULL, NULL}
};

/* Gets an attribute of a MatchObject. */
static PyObject* match_getattr(MatchObject* self, char* name) {
    PyObject* res;

    res = Py_FindMethod(match_methods, (PyObject*)self, name);
    if (res)
        return res;

    PyErr_Clear();

    if (!strcmp(name, "lastindex")) {
        if (self->lastindex > 0)
            return Py_BuildValue("i", self->lastindex);
        Py_INCREF(Py_None);
        return Py_None;
    }

    if (!strcmp(name, "lastgroup")) {
        if (self->pattern->indexgroup && self->lastgroup > 0) {
            PyObject* index = Py_BuildValue("i", self->lastgroup);
            PyObject* result = PyDict_GetItem(self->pattern->indexgroup, index);
            Py_DECREF(index);
            if (result)
                return result;
            PyErr_Clear();
        }
        Py_INCREF(Py_None);
        return Py_None;
    }

    if (!strcmp(name, "string")) {
        if (self->string) {
            Py_INCREF(self->string);
            return self->string;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }

    if (!strcmp(name, "regs")) {
        if (self->regs) {
            Py_INCREF(self->regs);
            return self->regs;
        } else
            return match_regs(self);
    }

    if (!strcmp(name, "re")) {
        Py_INCREF(self->pattern);
        return (PyObject*) self->pattern;
    }

    if (!strcmp(name, "pos"))
        return Py_BuildValue("i", self->pos);

    if (!strcmp(name, "endpos"))
        return Py_BuildValue("i", self->endpos);

    PyErr_SetString(PyExc_AttributeError, name);
    return NULL;
}

static PyMappingMethods match_as_mapping = {
    (lenfunc)match_length, /*mp_length*/
    (binaryfunc)match_subscript, /*mp_subscript*/
    0, /*mp_ass_subscript*/
};

/* FIXME: implement setattr("string", None) as a special case (to detach the
 * associated string, if any.
 */
static PyTypeObject Match_Type = {
    PyObject_HEAD_INIT(NULL)
    0,
    "_" RE_MODULE "." RE_MODULE_UPPER "_Match",
    sizeof(MatchObject),
    0,
    (destructor)match_dealloc, /*tp_dealloc*/
    0, /*tp_print*/
    (getattrfunc)match_getattr, /*tp_getattr*/
    0,                  /* tp_setattr */
    0,                  /* tp_compare */
    0,                  /* tp_repr */
    0,                  /* tp_as_number */
    0,                  /* tp_as_sequence */
    &match_as_mapping,  /* tp_as_mapping */
    0,                  /* tp_hash */
    0,                  /* tp_call */
    0,                  /* tp_str */
    0,                  /* tp_getattro */
    0,                  /* tp_setattro */
    0,                  /* tp_as_buffer */
    Py_TPFLAGS_DEFAULT,  /* tp_flags */
    0,                  /* tp_doc */
    0,                  /* tp_traverse */
    0,                  /* tp_clear */
    0,                  /* tp_richcompare */
    0,                  /* tp_weaklistoffset */
    0,                  /* tp_iter */
    0,                  /* tp_iternext */
    match_methods,      /* tp_methods */
};

/* Creates a new MatchObject. */
Py_LOCAL(PyObject*) pattern_new_match(PatternObject* pattern, RE_State* state,
  int status) {
    /* Create MatchObject (from state object). */
    if (status > 0) {
        MatchObject* match;
        int g;

        /* Create MatchObject. */
        match = PyObject_NEW(MatchObject, &Match_Type);
        if (!match)
            return NULL;

        match->marks = PyMem_MALLOC(pattern->group_count * 2 *
          sizeof(*match->marks));
        if (!match->marks) {
            set_error(RE_ERROR_MEMORY, NULL);
            Py_DECREF(match);
            return NULL;
        }

        match->pattern = pattern;
        match->string = state->string;
        Py_INCREF(match->pattern);
        Py_INCREF(match->string);

        match->regs = NULL;
        match->group_count = pattern->group_count;

        match->pos = text_offset(state, state->slice_start);
        match->endpos = text_offset(state, state->text_ptr);

        /* Fill in group slices. */
        for (g = 0; g < pattern->group_count; g++) {
            int m = g * 2;
            Py_ssize_t start = text_offset(state, state->marks[m]);
            Py_ssize_t end = text_offset(state, state->marks[m + 1]);
            if (0 <= start && start <= end) {
                match->marks[m] = start;
                match->marks[m + 1] = end;
            } else {
                match->marks[m] = -1;
                match->marks[m + 1] = -1;
            }
        }

        match->lastindex = state->lastindex;
        match->lastgroup = state->lastgroup;

        return (PyObject*)match;
    } else if (status == 0) {
        /* No match. */
        Py_INCREF(Py_None);
        return Py_None;
    }

    /* Internal error. */
    set_error(status, NULL);
    return NULL;
}

/* Gets the text of a capture group from a state. */
Py_LOCAL(PyObject*) state_get_group(RE_State* state, Py_ssize_t index, PyObject*
  string, BOOL empty) {
    Py_ssize_t m;
    Py_ssize_t start;
    Py_ssize_t end;

    m = index * 2 - 2;
    if (string != Py_None && index >= 1 && index <= state->pattern->group_count
      && state->marks[m] && state->marks[m + 1]) {
        start = text_offset(state, state->marks[m]);
        end = text_offset(state, state->marks[m + 1]);
    } else {
        if (empty)
            /* Want empty string. */
            start = end = 0;
        else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }

    return PySequence_GetSlice(string, start, end);
}

/* -------------------------------------------------------------------- */
/* scanner methods (experimental) */

/* ScannerObject's 'match' method. */
static PyObject* scanner_match(ScannerObject* self, PyObject* unused) {
    RE_State* state = &self->state;
    PyObject* match;
    int status;

    state->text_ptr = state->slice_start;

    if (state->charsize == 1)
        status = bmatch(state, FALSE);
    else
        status = umatch(state, FALSE);

    if (PyErr_Occurred())
        return NULL;

    match = pattern_new_match((PatternObject*)self->pattern, state, status);

    state->allow_zero_width = state->text_ptr != state->slice_start;
    state->slice_start = state->text_ptr;

    return match;
}

/* ScannerObject's 'search' method. */
static PyObject* scanner_search(ScannerObject* self, PyObject *unused) {
    RE_State* state = &self->state;
    PyObject* match;
    int status;

    state->text_ptr = state->slice_start;

    if (state->charsize == 1)
        status = bmatch(state, TRUE);
    else
        status = umatch(state, TRUE);

    if (PyErr_Occurred())
        return NULL;

    match = pattern_new_match((PatternObject*)self->pattern, state, status);

    state->allow_zero_width = state->text_ptr != state->slice_start;
    state->slice_start = state->text_ptr;

    return match;
}

/* ScannerObject's methods. */
static PyMethodDef scanner_methods[] = {
    {"match", (PyCFunction) scanner_match, METH_NOARGS},
    {"search", (PyCFunction) scanner_search, METH_NOARGS},
    {NULL, NULL}
};

/* Deallocates a ScannerObject. */
static void scanner_dealloc(ScannerObject* self) {
    state_fini(&self->state);
    Py_DECREF(self->pattern);
    PyObject_DEL(self);
}

/* Gets an attribute from a ScannerObject. */
static PyObject* scanner_getattr(ScannerObject* self, char* name) {
    PyObject* res;

    res = Py_FindMethod(scanner_methods, (PyObject*)self, name);
    if (res)
        return res;

    PyErr_Clear();

    /* Attributes. */
    if (!strcmp(name, "pattern")) {
        Py_INCREF(self->pattern);
        return self->pattern;
    }

    PyErr_SetString(PyExc_AttributeError, name);
    return NULL;
}

static PyTypeObject Scanner_Type = {
    PyObject_HEAD_INIT(NULL)
    0,
    "_" RE_MODULE "." RE_MODULE_UPPER "_Scanner",
    sizeof(ScannerObject),
    0,
    (destructor)scanner_dealloc, /*tp_dealloc*/
    0, /*tp_print*/
    (getattrfunc)scanner_getattr, /*tp_getattr*/
};

/* Creates a new ScannerObject. */
static PyObject* pattern_scanner(PatternObject* pattern, PyObject* args) {
    /* Create search state object. */
    ScannerObject* self;

    PyObject* string;
    Py_ssize_t start = 0;
    Py_ssize_t end = PY_SSIZE_T_MAX;
    if (!PyArg_ParseTuple(args, "O|nn:scanner", &string, &start, &end))
        return NULL;

    /* Create a scanner object. */
    self = PyObject_NEW(ScannerObject, &Scanner_Type);
    if (!self)
        return NULL;

    if (!state_init(&self->state, pattern, string, start, end)) {
        PyObject_DEL(self);
        return NULL;
    }

    Py_INCREF(pattern);
    self->pattern = (PyObject*) pattern;

    return (PyObject*) self;
}

/* PatternObject's 'match' method. */
static PyObject* pattern_match(PatternObject* self, PyObject* args, PyObject*
  kw) {
    PyObject* string;
    Py_ssize_t start = 0;
    Py_ssize_t end = PY_SSIZE_T_MAX;
    RE_State state;
    int status;
    PyObject* match;

    static char* kwlist[] = { "string", "pos", "endpos", NULL };
    if (!PyArg_ParseTupleAndKeywords(args, kw, "O|nn:match", kwlist, &string,
      &start, &end))
        return NULL;

    if (!state_init(&state, self, string, start, end))
        return NULL;

    if (state.charsize == 1)
        status = bmatch(&state, FALSE);
    else
        status = umatch(&state, FALSE);

    if (PyErr_Occurred()) {
        state_fini(&state);
        return NULL;
    }

    match = pattern_new_match(self, &state, status);

    state_fini(&state);

    return match;
}

/* PatternObject's 'search' method. */
static PyObject* pattern_search(PatternObject* self, PyObject* args, PyObject*
  kw) {
    PyObject* string;
    Py_ssize_t start = 0;
    Py_ssize_t end = PY_SSIZE_T_MAX;
    RE_State state;
    int status;
    PyObject* match;

    static char* kwlist[] = { "string", "pos", "endpos", NULL };
    if (!PyArg_ParseTupleAndKeywords(args, kw, "O|nn:search", kwlist, &string,
      &start, &end))
        return NULL;

    if (!state_init(&state, self, string, start, end))
        return NULL;

    if (state.charsize == 1)
        status = bmatch(&state, TRUE);
    else
        status = umatch(&state, TRUE);

    if (PyErr_Occurred()) {
        state_fini(&state);
        return NULL;
    }

    match = pattern_new_match(self, &state, status);

    state_fini(&state);

    return match;
}

/* Gets a replacement item from the replacement list.
 *
 * The replacement item could be a string literal or a group.
 */
PyObject* get_sub_replacement(PyObject* item, PyObject* string, RE_State* state,
  int group_count) {
    Py_ssize_t group;

    if (PyString_Check(item) || PyUnicode_Check(item)) {
        /* It's a literal, which can be added directly to the list. */
        Py_INCREF(item);
        return item;
    }

    /* Is it a group reference? */
    if (PyInt_Check(item) ||PyLong_Check(item) )
        group = PyInt_AsSsize_t(item);
    else {
        /* Not a group either! */
        set_error(RE_ERROR_REPLACEMENT, NULL);
        return NULL;
    }

    if (group == 0) {
        /* The entire matched portion of the string. */
        return PySequence_GetSlice(string, text_offset(state,
          state->slice_start), text_offset(state, state->text_ptr));
    } else if (group >= 1 && group <= group_count) {
        /* A group. If it didn't match then return None instead. */
        Py_ssize_t m = group * 2 - 2;
        if (state->marks[m])
            return PySequence_GetSlice(string, text_offset(state,
              state->marks[m]), text_offset(state, state->marks[m + 1]));
        else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    } else {
        /* No such group. */
        set_error(RE_ERROR_GROUP, NULL);
        return NULL;
    }
}

/* PatternObject's 'subx' method. */
Py_LOCAL(PyObject*) pattern_subx(PatternObject* self, PyObject* str_template,
  PyObject* string, Py_ssize_t count, Py_ssize_t subn) {
    PyObject* replacement;
    BOOL replacement_is_callable;
    RE_State state;
    PyObject* list;
    Py_ssize_t done;
    void* last;
    PyObject* item;

    /* sub/subn takes either a function or a string template. */
    replacement_is_callable = PyCallable_Check(str_template);
    if (replacement_is_callable) {
        /* The template is callable. */
        replacement = str_template;
        Py_INCREF(replacement);
    } else {
        /* The template isn't callable, so hand it over to the template
         * compiler.
         */
        replacement = call(RE_MODULE, "_compile_replacement", PyTuple_Pack(2,
          self, str_template));
        if (!replacement)
            return NULL;
    }

    if (!state_init(&state, self, string, 0, PY_SSIZE_T_MAX)) {
        Py_DECREF(replacement);
        return NULL;
    }

    list = PyList_New(0);
    if (!list) {
        Py_DECREF(replacement);
        state_fini(&state);
        return NULL;
    }

    done = 0;
    last = state.text_start;
    while (!count || done < count) {
        int status;

        if (state.charsize == 1)
            status = bmatch(&state, TRUE);
        else
            status = umatch(&state, TRUE);

        if (PyErr_Occurred())
            goto error;

        if (status == 0)
            break;

        if (status < 0) {
            set_error(status, NULL);
            goto error;
        }

        /* Append the segment before this match. */
        if (state.slice_start > last) {
            item = PySequence_GetSlice(string, text_offset(&state, last),
              text_offset(&state, state.slice_start));
            if (!item)
                goto error;

            /* Add to the list. */
            status = PyList_Append(list, item);
            Py_DECREF(item);
            if (status < 0) {
                set_error(status, NULL);
                goto error;
            }
        }

        /* Add this match. */
        if (replacement_is_callable) {
            /* Pass the MatchObject to the replacement function. */
            PyObject* match;
            PyObject* args;

            match = pattern_new_match(self, &state, 1);
            if (!match)
                goto error;
            args = PyTuple_Pack(1, match);
            if (!args) {
                Py_DECREF(match);
                goto error;
            }
            item = PyObject_CallObject(replacement, args);
            Py_DECREF(args);
            Py_DECREF(match);
            if (!item)
                goto error;

            /* Add to the list. */
            if (item != Py_None) {
                status = PyList_Append(list, item);
                Py_DECREF(item);
                if (status < 0) {
                    set_error(status, NULL);
                    goto error;
                }
            }
        } else {
            /* The replacement is a literal template. */
            Py_ssize_t size;
            Py_ssize_t i;

            /* Add each part of the template to the list. */
            size = PyList_GET_SIZE(replacement);
            for (i = 0; i < size; i++) {
                PyObject* item;
                PyObject* str_item;

                item = PyList_GET_ITEM(replacement, i);
                str_item = get_sub_replacement(item, string, &state,
                  self->group_count);
                if (!str_item)
                    goto error;

                /* Add to the list. */
                if (str_item == Py_None)
                    Py_DECREF(str_item);
                else {
                    status = PyList_Append(list, str_item);
                    Py_DECREF(str_item);
                    if (status < 0) {
                        set_error(status, NULL);
                        goto error;
                    }
                }
            }
        }

        ++done;

        /* Continue where we left off. */
        last = state.slice_start = state.text_ptr;
        state.allow_zero_width = FALSE;
    }

    /* Get the segment following the last match. */
    if (last < state.text_end) {
        int status;

        item = PySequence_GetSlice(string, text_offset(&state, last),
          text_offset(&state, state.text_end));
        if (!item)
            goto error;
        status = PyList_Append(list, item);
        Py_DECREF(item);
        if (status < 0) {
            set_error(status, NULL);
            goto error;
        }
    }

    state_fini(&state);

    Py_DECREF(replacement);

    /* Convert the list to a single string (also removes the list). */
    item = join_list(list, string);
    if (!item)
        return NULL;

    if (subn)
        return Py_BuildValue("Ni", item, done);

    return item;

error:
    Py_DECREF(list);
    state_fini(&state);
    Py_DECREF(replacement);
    return NULL;
}

/* PatternObject's 'sub' method. */
static PyObject* pattern_sub(PatternObject* self, PyObject* args, PyObject* kw)
  {
    PyObject* ptemplate;
    PyObject* string;
    Py_ssize_t count = 0;

    static char* kwlist[] = { "repl", "string", "count", NULL };
    if (!PyArg_ParseTupleAndKeywords(args, kw, "OO|n:sub", kwlist, &ptemplate,
      &string, &count))
        return NULL;

    return pattern_subx(self, ptemplate, string, count, 0);
}

/* PatternObject's 'subn' method. */
static PyObject* pattern_subn(PatternObject* self, PyObject* args, PyObject* kw)
  {
    PyObject* ptemplate;
    PyObject* string;
    Py_ssize_t count = 0;

    static char* kwlist[] = { "repl", "string", "count", NULL };
    if (!PyArg_ParseTupleAndKeywords(args, kw, "OO|n:subn", kwlist, &ptemplate,
      &string, &count))
        return NULL;

    return pattern_subx(self, ptemplate, string, count, 1);
}

/* PatternObject's 'split' method. */
static PyObject* pattern_split(PatternObject* self, PyObject* args, PyObject*
  kw) {
    RE_State state;
    PyObject* list;
    PyObject* item;
    int status;
    Py_ssize_t n;
    Py_ssize_t i;
    void* last;

    PyObject* string;
    Py_ssize_t maxsplit = 0;
    static char* kwlist[] = { "string", "maxsplit", NULL };
    if (!PyArg_ParseTupleAndKeywords(args, kw, "O|n:split", kwlist, &string,
      &maxsplit))
        return NULL;

    if (!state_init(&state, self, string, 0, PY_SSIZE_T_MAX))
        return NULL;

    list = PyList_New(0);
    if (!list) {
        state_fini(&state);
        return NULL;
    }

    n = 0;
    last = state.slice_start;

    while (!maxsplit || n < maxsplit) {
        if (state.charsize == 1)
            status = bmatch(&state, TRUE);
        else
            status = umatch(&state, TRUE);

        if (PyErr_Occurred())
            goto error;

        if (status == 0)
            /* No more matches. */
            break;

        if (status < 0) {
            set_error(status, NULL);
            goto error;
        }

        if (!state.zero_width) {
            /* The current behaviour is to advance one character if the split
             * was zero-width. Unfortunately, this can give an incorrect result.
             * GvR wants this behaviour to be retained so as not to break any
             * existing software which might rely on it. The correct behaviour
             * is enabled by setting the 'zero_width' flag.
             */
            if (state.text_ptr == state.slice_start) {
                if (last == state.slice_end)
                    break;

                /* Advance one character. */
                state.slice_start = (char*)state.text_ptr + state.charsize;
                state.allow_zero_width = TRUE;
                continue;
            }
        }

        /* Get segment before this match. */
        item = PySequence_GetSlice(string, text_offset(&state, last),
          text_offset(&state, state.slice_start));
        if (!item)
            goto error;
        status = PyList_Append(list, item);
        Py_DECREF(item);
        if (status < 0)
            goto error;

        /* Add groups (if any). */
        for (i = 1; i <= self->group_count; i++) {
            item = state_get_group(&state, i, string, FALSE);
            if (!item)
                goto error;
            status = PyList_Append(list, item);
            Py_DECREF(item);
            if (status < 0)
                goto error;
        }

        ++n;

        /* Continue from where we left off. */
        last = state.slice_start = state.text_ptr;

        /* We don't want a zero-width match right after a split. */
        state.allow_zero_width = FALSE;
    }

    /* Get segment following last match (even if empty). */
    item = PySequence_GetSlice(string, text_offset(&state, last),
      text_offset(&state, state.slice_end));
    if (!item)
        goto error;
    status = PyList_Append(list, item);
    Py_DECREF(item);
    if (status < 0)
        goto error;

    state_fini(&state);
    return list;

error:
    Py_DECREF(list);
    state_fini(&state);
    return NULL;
}

/* PatternObject's 'findall' method. */
static PyObject* pattern_findall(PatternObject* self, PyObject* args, PyObject*
  kw) {
    RE_State state;
    PyObject* list;
    int status;
    Py_ssize_t i, b, e;

    PyObject* string;
    Py_ssize_t start = 0;
    Py_ssize_t end = PY_SSIZE_T_MAX;
    static char* kwlist[] = { "string", "pos", "endpos", NULL };
    if (!PyArg_ParseTupleAndKeywords(args, kw, "O|nn:findall", kwlist, &string,
      &start, &end))
        return NULL;

    if (!state_init(&state, self, string, start, end))
        return NULL;

    list = PyList_New(0);
    if (!list) {
        state_fini(&state);
        return NULL;
    }

    while (state.slice_start <= state.slice_end) {
        PyObject* item;

        state.text_ptr = state.slice_start;

        if (state.charsize == 1)
            status = bmatch(&state, TRUE);
        else
            status = umatch(&state, TRUE);

        if (PyErr_Occurred())
            goto error;

        if (status == 0)
            break;

        if (status < 0) {
            set_error(status, NULL);
            goto error;
        }

        /* Don't bother to build a MatchObject. */
        switch (self->group_count) {
        case 0:
            b = text_offset(&state, state.slice_start);
            e = text_offset(&state, state.text_ptr);
            item = PySequence_GetSlice(string, b, e);
            if (!item)
                goto error;
            break;
        case 1:
            item = state_get_group(&state, 1, string, TRUE);
            if (!item)
                goto error;
            break;
        default:
            item = PyTuple_New(self->group_count);
            if (!item)
                goto error;
            for (i = 0; i < self->group_count; i++) {
                PyObject* o = state_get_group(&state, i + 1, string, TRUE);
                if (!o) {
                    Py_DECREF(item);
                    goto error;
                }
                PyTuple_SET_ITEM(item, i, o);
            }
            break;
        }

        status = PyList_Append(list, item);
        Py_DECREF(item);
        if (status < 0)
            goto error;

        state.slice_start = state.text_ptr;
    }

    state_fini(&state);
    return list;

error:
    Py_DECREF(list);
    state_fini(&state);
    return NULL;
}

/* PatternObject's 'finditer' method. */
static PyObject* pattern_finditer(PatternObject* pattern, PyObject* args) {
    PyObject* scanner;
    PyObject* search;
    PyObject* iterator;

    scanner = pattern_scanner(pattern, args);
    if (!scanner)
        return NULL;

    search = PyObject_GetAttrString(scanner, "search");
    Py_DECREF(scanner);
    if (!search)
        return NULL;

    iterator = PyCallIter_New(search, Py_None);
    Py_DECREF(search);

    return iterator;
}

/* PatternObject's 'copy' method. */
static PyObject* pattern_copy(PatternObject* self, PyObject *unused) {
    PyErr_SetString(PyExc_TypeError, "cannot copy this PatternObject");
    return NULL;
}

/* PatternObject's 'deepcopy' method. */
static PyObject* pattern_deepcopy(PatternObject* self, PyObject* memo) {
    PyErr_SetString(PyExc_TypeError, "cannot deepcopy this PatternObject");
    return NULL;
}

/* The documentation of a PatternObject. */
PyDoc_STRVAR(pattern_match_doc,
    "match(string[, pos[, endpos]]) --> MatchObject or None.\n\
    Matches zero or more characters at the beginning of the string");

PyDoc_STRVAR(pattern_search_doc,
    "search(string[, pos[, endpos]]) --> MatchObject or None.\n\
    Scan through string looking for a match, and return a corresponding\n\
    MatchObject instance. Return None if no position in the string matches.");

PyDoc_STRVAR(pattern_sub_doc,
    "sub(repl, string[, count = 0]) --> newstring\n\
    Return the string obtained by replacing the leftmost non-overlapping\n\
    occurrences of pattern in string by the replacement repl.");

PyDoc_STRVAR(pattern_subn_doc,
    "subn(repl, string[, count = 0]) --> (newstring, number of subs)\n\
    Return the tuple (new_string, number_of_subs_made) found by replacing\n\
    the leftmost non-overlapping occurrences of pattern with the\n\
    replacement repl.");

PyDoc_STRVAR(pattern_split_doc,
    "split(string[, maxsplit = 0])  --> list.\n\
    Split string by the occurrences of pattern.");

PyDoc_STRVAR(pattern_findall_doc,
    "findall(string[, pos[, endpos]]) --> list.\n\
    Return a list of all non-overlapping matches of pattern in string.");

PyDoc_STRVAR(pattern_finditer_doc,
    "finditer(string[, pos[, endpos]]) --> iterator.\n\
    Return an iterator over all non-overlapping matches for the \n\
    RE pattern in string. For each match, the iterator returns a\n\
    MatchObject.");

/* The methods of a PatternObject. */
static PyMethodDef pattern_methods[] = {
    {"match", (PyCFunction)pattern_match, METH_VARARGS|METH_KEYWORDS,
      pattern_match_doc},
    {"search", (PyCFunction)pattern_search, METH_VARARGS|METH_KEYWORDS,
      pattern_search_doc},
    {"sub", (PyCFunction)pattern_sub, METH_VARARGS|METH_KEYWORDS,
      pattern_sub_doc},
    {"subn", (PyCFunction)pattern_subn, METH_VARARGS|METH_KEYWORDS,
      pattern_subn_doc},
    {"split", (PyCFunction)pattern_split, METH_VARARGS|METH_KEYWORDS,
      pattern_split_doc},
    {"findall", (PyCFunction)pattern_findall, METH_VARARGS|METH_KEYWORDS,
      pattern_findall_doc},
    {"finditer", (PyCFunction)pattern_finditer, METH_VARARGS,
      pattern_finditer_doc},
    {"scanner", (PyCFunction)pattern_scanner, METH_VARARGS},
    {"__copy__", (PyCFunction)pattern_copy, METH_NOARGS},
    {"__deepcopy__", (PyCFunction)pattern_deepcopy, METH_O},
    {NULL, NULL}
};

PyDoc_STRVAR(pattern_doc, "Compiled regular expression objects");

/* Deallocates a PatternObject. */
static void pattern_dealloc(PatternObject* self) {
    int i;

    /* Discard the nodes. */
    for (i = 0; i < self->node_count; i ++)
        PyMem_FREE(self->node_list[i]);
    PyMem_FREE(self->node_list);

    /* Discard the group info. */
    PyMem_FREE(self->group_info_list);

    if (self->weakreflist)
        PyObject_ClearWeakRefs((PyObject*)self);
    Py_XDECREF(self->pattern);
    PyObject_DEL(self);
}

/* Gets an attribute of a PatternObject. */
static PyObject* pattern_getattr(PatternObject* self, char* name) {
    PyObject* res;

    res = Py_FindMethod(pattern_methods, (PyObject*) self, name);
    if (res)
        return res;

    PyErr_Clear();

    /* Attributes. */
    if (!strcmp(name, "pattern")) {
        Py_INCREF(self->pattern);
        return self->pattern;
    }

    if (!strcmp(name, "flags"))
        return Py_BuildValue("i", self->flags);

    if (!strcmp(name, "groups"))
        return Py_BuildValue("i", self->group_count);

    if (!strcmp(name, "groupindex") && self->groupindex) {
        Py_INCREF(self->groupindex);
        return self->groupindex;
    }

    if (!strcmp(name, "groupcount")) {
        PyObject* result;

        result = Py_BuildValue("i", self->group_count);
        return result;
    }

    PyErr_SetString(PyExc_AttributeError, name);
    return NULL;
}

static PyTypeObject Pattern_Type = {
    PyObject_HEAD_INIT(NULL)
    0,
    "_" RE_MODULE "." RE_MODULE_UPPER "_Pattern",
    sizeof(PatternObject),
    0,
    (destructor)pattern_dealloc, /*tp_dealloc*/
    0, /*tp_print*/
    (getattrfunc)pattern_getattr, /*tp_getattr*/
    0,                  /* tp_setattr */
    0,                  /* tp_compare */
    0,                  /* tp_repr */
    0,                  /* tp_as_number */
    0,                  /* tp_as_sequence */
    0,                  /* tp_as_mapping */
    0,                  /* tp_hash */
    0,                  /* tp_call */
    0,                  /* tp_str */
    0,                  /* tp_getattro */
    0,                  /* tp_setattro */
    0,                  /* tp_as_buffer */
    Py_TPFLAGS_HAVE_WEAKREFS,       /* tp_flags */
    pattern_doc,            /* tp_doc */
    0,                  /* tp_traverse */
    0,                  /* tp_clear */
    0,                  /* tp_richcompare */
    offsetof(PatternObject, weakreflist),   /* tp_weaklistoffset */
};

/* Building the nodes is made simpler by allowing branches to have a single
 * exit. These need to be removed.
 */
Py_LOCAL(void) skip_one_way_branches(PatternObject* pattern) {
    BOOL modified;

    /* The start node might be a 1-way branch. Skip over it because it'll be
     * removed. It might even be the first in a chain.
     */
    while (pattern->start_node->op == RE_OP_BRANCH &&
      !pattern->start_node->next_2)
        pattern->start_node = pattern->start_node->next_1;

    /* If a node refers to a 1-way branch then make the former refer to the
     * latter's destination. Repeat until they're all done.
     */
    do {
        int i;

        modified = FALSE;

        for(i = 0; i < pattern->node_count; i++) {
            RE_PatternNode* node;
            RE_PatternNode* next;

            node = pattern->node_list[i];

            /* Check the first destination. */
            next = node->next_1;
            if (next && next->op == RE_OP_BRANCH && !next->next_2) {
                node->next_1 = next->next_1;
                modified = TRUE;
            }

            /* Check the second destination. */
            next = node->next_2;
            if (next && next->op == RE_OP_BRANCH && !next->next_2) {
                node->next_2 = next->next_1;
                modified = TRUE;
            }
        }
    } while (modified);
}

/* Discards any unused nodes.
 *
 * Optimising the nodes might result in some nodes no longer being used.
 */
Py_LOCAL(void) discard_unused_nodes(PatternObject* pattern) {
    Py_ssize_t new_count;
    Py_ssize_t old_count;
    int i;

    /* The start node is used. */
    new_count = 0;
    pattern->start_node->index = new_count++;

    /* If a node is used then all the nodes to which it refers are also used.
     * Repeat until they're all done.
     */
    do {
        old_count = new_count;

        for (i = 0; i < pattern->node_count; i++) {
            RE_PatternNode* node;

            node = pattern->node_list[i];
            if (node->index >= 0) {
                RE_PatternNode* next;

                /* Check the first destination. */
                next = node->next_1;
                if (next && next->index < 0)
                    next->index = new_count++;

                /* Check the second destination. */
                next = node->next_2;
                if(next && next->index < 0)
                    next->index = new_count++;
            }
        }
    } while (new_count > old_count);

    /* Remove the nodes which aren't used. */
    new_count = 0;

    for (i = 0; i < pattern->node_count; i++) {
        RE_PatternNode* node;

        node = pattern->node_list[i];
        if (node->index >= 0)
            pattern->node_list[new_count++] = node;
        else
            PyMem_FREE(node);
    }

    pattern->node_count = new_count;
}

/* Assigns offsets for the groups into the pattern->values array.
 *
 * Entries for groups which are referred to come first, just after the entries
 * for the repeats.
 */
Py_LOCAL(BOOL) assign_group_offsets(PatternObject* pattern) {
    int ref_offset;
    int nonref_offset;
    int i;

    /* For simplicity we'll assign offsets for referenced groups upwards and for
     * non-referenced groups downwards in the pattern->values array.
     */
    ref_offset = pattern->repeat_count;
    nonref_offset = pattern->repeat_count + pattern->group_count;

    for (i = 0; i < pattern->group_count; i++) {
        RE_GroupInfo* group_info;

        group_info = &pattern->group_info_list[i];
        if (group_info->id < 1) {
            /* There's an entry for a non-existant group. */
            set_error(RE_ERROR_ILLEGAL, NULL);
            return FALSE;
        }

        if (group_info->referenced)
            group_info->value_offset = ref_offset++;
        else
            group_info->value_offset = --nonref_offset;
    }

    /* The number of groups which are referenced. */
    pattern->ref_group_count = ref_offset - pattern->repeat_count;

    return TRUE;
}

/* Marks all the group which are named. */
Py_LOCAL(void) mark_named_groups(PatternObject* pattern) {
    int i;

    for (i = 0; i < pattern->group_count; i++) {
        RE_GroupInfo* group_info;
        PyObject* index;

        group_info = &pattern->group_info_list[i];
        index = Py_BuildValue("i", i + 1);
        group_info->has_name = PyDict_Contains(pattern->indexgroup, index);
        Py_DECREF(index);
    }
}

/* Destroy all the nodes.
 *
 * The PatternObject contains a list of all the nodes.
 */
Py_LOCAL(void) destroy_nodes(PatternObject* pattern) {
    RE_PatternNode** node_list;
    int i;

    node_list = pattern->node_list;
    for (i = 0; i < pattern->node_count; i++) {
        PyMem_FREE(node_list[i]->values);
        PyMem_FREE(node_list[i]);
    }

    pattern->node_capacity = 0;
    pattern->node_count = 0;
    PyMem_FREE(pattern->node_list);
    pattern->node_list = NULL;
}

#if defined(VERBOSE)
static int compare_nodes(const void* elem1, const void* elem2) {
    RE_PatternNode** e1 = (RE_PatternNode**)elem1;
    RE_PatternNode** e2 = (RE_PatternNode**)elem2;
    return (int)((*e1)->index - (*e2)->index);
}

#endif
/* Optimises the pattern. */
Py_LOCAL(BOOL) optimise_pattern(PatternObject* pattern) {
    /* Building the nodes is made simpler by allowing branches to have a single
     * exit. These need to be removed.
     */
    skip_one_way_branches(pattern);

    /* Discard any unused nodes. */
    discard_unused_nodes(pattern);

#if defined(VERBOSE)
    {
        RE_PatternNode** sorted_nodes = PyMem_MALLOC(pattern->node_count *
          sizeof(*sorted_nodes));
        int i;
        for (i = 0; i < pattern->node_count; i ++)
            sorted_nodes[i] = pattern->node_list[i];
        qsort(sorted_nodes, pattern->node_count, sizeof(*sorted_nodes),
          compare_nodes);
        for (i = 0; i < pattern->node_count; i ++) {
            RE_PatternNode* node = sorted_nodes[i];
            printf("node->index is %d, node->op is %s", node->index,
              re_op_text[node->op]);
            if (node->next_1)
                printf(", node->next_1 is %d", node->next_1->index);
            if (node->next_2)
                printf(", node->next_2 is %d", node->next_2->index);
            printf("\n");
        }
        PyMem_FREE(sorted_nodes);
    }

#endif
    /* Assign group offsets into the pattern->values array.
     *
     * Entries for groups which are referred to elsewhere in the regular
     * expression come first, just after the entries for the repeats.
     *
     * This is an internal optimisation for the core of the matching engine.
     */
    if (!assign_group_offsets(pattern)) {
        destroy_nodes(pattern);
        return FALSE;
    }

    /* Mark all the group that are named. */
    mark_named_groups(pattern);

    /* The repeat counts and the referenced groups are stored in one block.
     * Calculate the size of the block.
     */
    pattern->compare_size = (pattern->repeat_count + pattern->ref_group_count) *
      sizeof(RE_ContextValue);

    return TRUE;
}

/* Creates a new pattern node. */
Py_LOCAL(RE_PatternNode*) create_node(PatternObject* pattern, RE_CODE op, int
  value_count) {
    RE_PatternNode* node;

    node = (RE_PatternNode*)PyMem_MALLOC(sizeof(*node));
    if (!node) {
        set_error(RE_ERROR_MEMORY, NULL);
        return NULL;
    }
    memset(node, 0, sizeof(*node));

    node->value_capacity = value_count;
    node->value_count = value_count;
    node->values = PyMem_MALLOC(node->value_capacity * sizeof(*node->values));
    if (!node->values) {
        set_error(RE_ERROR_MEMORY, NULL);
        PyMem_FREE(node);
        return NULL;
    }

    node->index = -1;
    node->op = op;

    /* Ensure that there's enough storage to record the new node. */
    if (pattern->node_count >= pattern->node_capacity) {
        RE_PatternNode** new_node_list;

        pattern->node_capacity += 16;
        new_node_list = (RE_PatternNode**)PyMem_REALLOC(pattern->node_list,
          pattern->node_capacity * sizeof(*new_node_list));
        if (!new_node_list) {
            set_error(RE_ERROR_MEMORY, NULL);
            PyMem_FREE(node);
            return NULL;
        }
        pattern->node_list = new_node_list;
    }

    /* Record the new node. */
    pattern->node_list[pattern->node_count++] = node;

    return node;
}

/* Adds a node as a next node for another node. */
Py_LOCAL(void) add_node(RE_PatternNode* node_1, RE_PatternNode* node_2) {
    if (!node_1->next_1)
        node_1->next_1 = node_2;
    else
        node_1->next_2 = node_2;
}

/* Ensures that the entry for a group's details actually exists. */
Py_LOCAL(BOOL) ensure_group(PatternObject* pattern, int group) {
    Py_ssize_t old_capacity;
    Py_ssize_t new_capacity;
    RE_GroupInfo* new_group_info_list;

    if (group <= pattern->group_count)
        /* We already have an entry for the group. */
        return TRUE;

    /* Increase the storage capacity to include the new entry. */
    old_capacity = pattern->group_info_capacity;
    new_capacity = pattern->group_info_capacity;
    while (group > new_capacity)
        new_capacity += 16;

    new_group_info_list = (RE_GroupInfo*)PyMem_REALLOC(pattern->group_info_list,
      new_capacity * sizeof(*new_group_info_list));
    if (!new_group_info_list) {
        set_error(RE_ERROR_MEMORY, NULL);
        return FALSE;
    }
    memset(new_group_info_list + old_capacity, 0, (new_capacity - old_capacity)
      * sizeof(*new_group_info_list));

    pattern->group_info_list = new_group_info_list;
    pattern->group_info_capacity = new_capacity;
    pattern->group_count = group;

    return TRUE;
}

/* Records that there's a reference to a group. */
Py_LOCAL(BOOL) record_ref_group(PatternObject* pattern, int group) {
    if (!ensure_group(pattern, group))
        return FALSE;

    pattern->group_info_list[group - 1].referenced = TRUE;

    return TRUE;
}

/* Records that there's a new group. */
Py_LOCAL(BOOL) record_group(PatternObject* pattern, int group) {
    if (!ensure_group(pattern, group))
        return FALSE;

    if (group >= 1)
        pattern->group_info_list[group - 1].id = group;

    return TRUE;
}

/* Adds a member to a character set. */
Py_LOCAL(BOOL) add_set_member(RE_PatternNode* node, RE_CODE op, int arg_count) {
    arg_count = 1 + arg_count;
    if (node->value_count + arg_count > node->value_capacity) {
        unsigned int* new_values;

        node->value_capacity += arg_count;
        if (node->value_capacity < 16)
            node->value_capacity = 16;
        new_values = (unsigned int*)PyMem_REALLOC(node->values,
          node->value_capacity * sizeof(*new_values));
        if (!new_values) {
            set_error(RE_ERROR_MEMORY, NULL);
            return FALSE;
        }
        node->values = new_values;
    }

    node->values[node->value_count++] = op;

    return TRUE;
}

Py_LOCAL(BOOL) build_sequence(RE_CODE** code, RE_CODE* end_code,
  RE_PatternNode** start, RE_PatternNode** end, PatternObject* pattern);

/* Builds a simple operation. */
Py_LOCAL(BOOL) build_simple(RE_CODE op, unsigned int arg, RE_PatternNode**
  start_ptr, RE_PatternNode** end_ptr, PatternObject* pattern) {
    RE_PatternNode* node;

    /* Create the node. */
    node = create_node(pattern, op, 1);
    if (!node)
        return FALSE;
    node->values[0] = arg;

    /* Append the node. */
    add_node(*end_ptr, node);
    *end_ptr = node;

    return TRUE;
}

/* Builds an optional item. */
Py_LOCAL(BOOL) build_optional(PatternObject* pattern, RE_CODE** code_ptr,
  RE_CODE* end_code, RE_PatternNode** start_ptr, RE_PatternNode** end_ptr, BOOL
  greedy) {
    RE_PatternNode* branch_node;
    RE_PatternNode* join_node;
    RE_PatternNode* rep_start;
    RE_PatternNode* rep_end;

    /* Create the start and end nodes. */
    branch_node = create_node(pattern, RE_OP_BRANCH, 0);
    join_node = create_node(pattern, RE_OP_BRANCH, 0);
    if (!branch_node || !join_node)
        return FALSE;

    /* Compile the item and check that that we've reached the end of it. */
    if (!build_sequence(code_ptr, end_code, &rep_start, &rep_end, pattern))
        return FALSE;

    if ((*code_ptr)[0] != RE_OP_END) {
        set_error(RE_ERROR_ILLEGAL, NULL);
        return FALSE;
    }
    ++(*code_ptr);

    if (greedy) {
        /* It's a greedy option. */
        add_node(branch_node, rep_start);
        add_node(branch_node, join_node);
    } else {
        /* It's a lazy option. */
        add_node(branch_node, join_node);
        add_node(branch_node, rep_start);
    }
    add_node(rep_end, join_node);

    /* Append the start node. */
    add_node(*end_ptr, branch_node);
    *end_ptr = join_node;

    return TRUE;
}

/* Builds an unlimited repeat. */
Py_LOCAL(BOOL) build_limited_repeat(PatternObject* pattern, RE_CODE** code_ptr,
  RE_CODE* end_code, RE_PatternNode** start_ptr, RE_PatternNode** end_ptr, BOOL
  greedy, RE_CODE min_count, RE_CODE max_count) {
    RE_PatternNode* repeat_node;
    RE_PatternNode* rep_start;
    RE_PatternNode* rep_end;

    if (min_count == 1 && max_count == 1) {
        /* Only a single repeat of the 'body', so compile just the body and
         * check that we've reached the end of it.
         */
        if (!build_sequence(code_ptr, end_code, &rep_start, &rep_end, pattern))
            return FALSE;

        if ((*code_ptr)[0] != RE_OP_END) {
            set_error(RE_ERROR_ILLEGAL, NULL);
            return FALSE;
        }
        ++(*code_ptr);

        /* Append the body. */
        add_node(*end_ptr, rep_start);
        *end_ptr = rep_end;
    } else {
        /* Create a node for the repeat. */
        repeat_node = create_node(pattern, greedy ? RE_OP_GREEDY_REPEAT :
          RE_OP_LAZY_REPEAT, 3);
        if (!repeat_node)
            return FALSE;
        repeat_node->values[0] = pattern->repeat_count++;
        repeat_node->values[1] = min_count;
        repeat_node->values[2] = max_count;

        /* Compile the 'body' and check that we've reached the end of it. */
        if (!build_sequence(code_ptr, end_code, &rep_start, &rep_end, pattern))
            return FALSE;

        if ((*code_ptr)[0] != RE_OP_END) {
            set_error(RE_ERROR_ILLEGAL, NULL);
            return FALSE;
        }
        ++(*code_ptr);

        /* Insert the body. */
        add_node(repeat_node, rep_start);
        add_node(rep_end, repeat_node);

        /* Append the repeat node. */
        add_node(*end_ptr, repeat_node);
        *end_ptr = repeat_node;
    }

    return TRUE;
}

/* Builds an unlimited repeat. */
Py_LOCAL(BOOL) build_unlimited_repeat(PatternObject* pattern, RE_CODE**
  code_ptr, RE_CODE* end_code, RE_PatternNode** start_ptr, RE_PatternNode**
  end_ptr, BOOL greedy) {
    RE_PatternNode* repeat_node;
    RE_PatternNode* rep_start;
    RE_PatternNode* rep_end;

    /* Create a node for the repeat. */
    repeat_node = create_node(pattern, greedy ? RE_OP_UNLIMITED_GREEDY_REPEAT :
      RE_OP_UNLIMITED_LAZY_REPEAT, 1);
    if (!repeat_node)
        return FALSE;
    repeat_node->values[0] = pattern->repeat_count++;

    /* Compile the 'body' of the repeat and check that we've reached the end of
     * the body.
     */
    if (!build_sequence(code_ptr, end_code, &rep_start, &rep_end, pattern))
        return FALSE;

    if ((*code_ptr)[0] != RE_OP_END) {
        set_error(RE_ERROR_ILLEGAL, NULL);
        return FALSE;
    }
    ++(*code_ptr);

    /* Insert the body. */
    add_node(repeat_node, rep_start);
    add_node(rep_end, repeat_node);

    /* Append the repeat node. */
    add_node(*end_ptr, repeat_node);
    *end_ptr = repeat_node;

    return TRUE;
}

/* Builds a sequence of nodes from regular expression code. */
Py_LOCAL(BOOL) build_sequence(RE_CODE** code, RE_CODE* end_code,
  RE_PatternNode** start, RE_PatternNode** end, PatternObject* pattern) {
    /* Guarantee that there's something to attach to. */
    *start = create_node(pattern, RE_OP_BRANCH, 0);
    *end = *start;

    /* The sequence should end with an opcode we don't understand. If it doesn't
     * then the code is illegal.
     */
    while (*code < end_code) {
        RE_CODE op;

        /* The following code groups opcodes by format, not function. */
        op = (*code)[0];
        switch (op) {
        case RE_OP_ANY:
        case RE_OP_ANY_ALL:
        case RE_OP_ANY_REV:
        case RE_OP_ANY_ALL_REV:
        case RE_OP_END_OF_LINE:
        case RE_OP_END_OF_STRING:
        case RE_OP_END_OF_STRING_LINE:
        case RE_OP_START_OF_LINE:
        case RE_OP_START_OF_STRING:
            /* A simple opcode with no trailing codewords. */
            if (!build_simple(op, 0, start, end, pattern))
                return FALSE;

            ++(*code);
            break;
        case RE_OP_ATOMIC:
        {
            /* An atomic subpattern. */
            RE_PatternNode* node;
            RE_PatternNode* start_sub;
            RE_PatternNode* end_sub;
            RE_PatternNode* success_node;

            node = create_node(pattern, RE_OP_ATOMIC, 0);
            if (!node)
                return FALSE;
            ++(*code);

            /* Compile the sequence and check that we've reached the end of the
             * subpattern.
             */
            if (!build_sequence(code, end_code, &start_sub, &end_sub, pattern))
                return FALSE;

            if ((*code)[0] != RE_OP_END) {
                set_error(RE_ERROR_ILLEGAL, NULL);
                return FALSE;
            }
            ++(*code);

            /* Create the success node to terminate the subpattern. */
            success_node = create_node(pattern, RE_OP_SUCCESS, 0);
            if (!success_node)
                return FALSE;

            /* Insert the subpattern. */
            node->next_2 = start_sub;
            add_node(end_sub, success_node);

            /* Append the atomic node. */
            add_node(*end, node);
            *end = node;
            break;
        }
        case RE_OP_BOUNDARY:
        {
            /* Word boundary. */
            RE_CODE positive;
            RE_PatternNode* node;

            positive = (*code)[1];
            if (positive != 0 && positive != 1) {
                set_error(RE_ERROR_ILLEGAL, NULL);
                return FALSE;
            }

            node = create_node(pattern, op, 1);
            if (!node)
                return FALSE;
            node->values[0] = positive;
            *code += 2;

            /* Append the node. */
            add_node(*end, node);
            *end = node;
            break;
        }
        case RE_OP_BRANCH:
        {
            /* A 2-way branch. */
            RE_PatternNode* branch_node;
            RE_PatternNode* join_node;

            /* Create nodes for the start and end of the branch sequence. */
            branch_node = create_node(pattern, RE_OP_BRANCH, 0);
            join_node = create_node(pattern, RE_OP_BRANCH, 0);
            if (!branch_node || !join_node)
                return FALSE;

            /* Append the start node. */
            add_node(*end, branch_node);
            *end = join_node;

            /* A branch in the regular expression is compiled into a series of
             * 2-way branches.
             */
            do {
                RE_PatternNode* seq_start;
                RE_PatternNode* seq_end;
                RE_PatternNode* next_branch_node;

                /* Skip over the 'BRANCH' or 'NEXT_BRANCH' opcode. */
                ++(*code);

                /* Compile the sequence until the next 'BRANCH' or 'NEXT_BRANCH'
                 * opcode.
                 */
                if (!build_sequence(code, end_code, &seq_start, &seq_end,
                  pattern))
                    return FALSE;

                /* Append the sequence. */
                add_node(branch_node, seq_start);
                add_node(seq_end, join_node);

                /* Create a start node for the next sequence and append it. */
                next_branch_node = create_node(pattern, RE_OP_BRANCH, 0);
                if (!next_branch_node)
                    return FALSE;

                add_node(branch_node, next_branch_node);
                branch_node = next_branch_node;
            } while (*code < end_code && (*code)[0] == RE_OP_NEXT);

            /* We should have reached the end of the branch. */
            if ((*code)[0] != RE_OP_END) {
                set_error(RE_ERROR_ILLEGAL, NULL);
                return FALSE;
            }
            ++(*code);
            break;
        }
        case RE_OP_CATEGORY:
        case RE_OP_CATEGORY_REV:
        {
            /* A character category. */
            RE_CODE positive;
            RE_CODE value;
            RE_PatternNode* node;

            positive = (*code)[1];
            value = (*code)[2];
            if (positive != 0 && positive != 1) {
                set_error(RE_ERROR_ILLEGAL, NULL);
                return FALSE;
            }

            node = create_node(pattern, op, 2);
            if (!node)
                return FALSE;
            node->values[0] = positive;
            node->values[1] = value;
            *code += 3;

            /* Append the category. */
            add_node(*end, node);
            *end = node;
            break;
        }
        case RE_OP_CHARACTER:
        case RE_OP_CHARACTER_IGNORE:
        case RE_OP_CHARACTER_REV:
        case RE_OP_CHARACTER_IGNORE_REV:
            /* A character literal. */
            if (!build_simple(op, (*code)[1], start, end, pattern))
                return FALSE;

            *code += 2;
            break;
        case RE_OP_GREEDY_REPEAT:
        case RE_OP_LAZY_REPEAT:
        {
            /* A repeated item. */
            BOOL greedy;
            RE_CODE min_count;
            RE_CODE max_count;

            /* This includes special cases such as optional items and unlimited
             * repeats, which we'll check for and treat specially. They don't
             * need repeat counts, which helps us avoid unnecessary work when
             * matching.
             */
            greedy = op == RE_OP_GREEDY_REPEAT;
            min_count = (*code)[1];
            max_count = (*code)[2];
            if (min_count > max_count) {
                set_error(RE_ERROR_ILLEGAL, NULL);
                return FALSE;
            }

            *code += 3;

            if (min_count == 0 && max_count == 1) {
                /* Optional item. */
                if (!build_optional(pattern, code, end_code, start, end,
                  greedy))
                    return FALSE;
            } else if (max_count == RE_UNLIMITED) {
                /* There's no upper limit, so split the repeat into a limited
                 * repeat up to the minimum followed by an unlimited repeat.
                 *
                 * For simplicity we'll just compile the repeated code twice.
                 */

                if (min_count > 0) {
                    RE_CODE* saved_code;

                    /* Save the position. */
                    saved_code = *code;

                    /* Compile the limited repeat up to the minimum. */
                    if (!build_limited_repeat(pattern, code, end_code, start,
                      end, greedy, min_count, min_count))
                        return FALSE;

                    /* Restore the position. */
                    *code = saved_code;
                }

                /* Compile the unlimited repeat. */
                if (!build_unlimited_repeat(pattern, code, end_code, start, end,
                  greedy))
                    return FALSE;
            } else {
                /* There's an upper limit, so compile an limited repeat. */
                if (!build_limited_repeat(pattern, code, end_code, start, end,
                  greedy, min_count, max_count))
                    return FALSE;
            }
            break;
        }
        case RE_OP_GROUP:
        {
            /* A capture group. */
            RE_CODE group;
            RE_PatternNode* start_node;
            RE_PatternNode* end_node;
            RE_PatternNode* start_sub;
            RE_PatternNode* end_sub;

            group = (*code)[1];
            *code += 2;

            /* Create nodes for the start and end of the capture group. */
            start_node = create_node(pattern, RE_OP_BEGIN_GROUP, 1);
            end_node = create_node(pattern, RE_OP_END_GROUP, 1);
            if (!start_node || !end_node)
                return FALSE;
            start_node->values[0] = group;
            end_node->values[0] = group;

            /* Record that we have a new capture group. */
            if (!record_group(pattern, group))
                return FALSE;

            /* Compile the sequence and check that we've reached the end of the
             * capture group.
             */
            if (!build_sequence(code, end_code, &start_sub, &end_sub, pattern))
                return FALSE;

            if ((*code)[0] != RE_OP_END) {
                set_error(RE_ERROR_ILLEGAL, NULL);
                return FALSE;
            }
            ++(*code);

            /* Append the capture group. */
            add_node(*end, start_node);
            add_node(start_node, start_sub);
            add_node(end_sub, end_node);
            *end = end_node;
            break;
        }
        case RE_OP_GROUP_EXISTS:
        {
            /* Checks whether a capture group is defined. */
            RE_CODE group;
            RE_PatternNode* start_node;
            RE_PatternNode* end_node;
            RE_PatternNode* start_sub;
            RE_PatternNode* end_sub;

            group = (*code)[1];
            *code += 2;

            /* Create nodes for the start and end of the structure. */
            start_node = create_node(pattern, RE_OP_GROUP_EXISTS, 1);
            end_node = create_node(pattern, RE_OP_BRANCH, 0);
            if (!start_node || !end_node)
                return FALSE;
            start_node->values[0] = group;

            if (!build_sequence(code, end_code, &start_sub, &end_sub, pattern))
                return FALSE;

            /* Append the start node. */
            add_node(*end, start_node);
            add_node(start_node, start_sub);
            add_node(end_sub, end_node);

            if ((*code)[0] == RE_OP_NEXT) {
                ++(*code);

                if (!build_sequence(code, end_code, &start_sub, &end_sub,
                  pattern))
                    return FALSE;

                add_node(start_node, start_sub);
                add_node(end_sub, end_node);
            } else
                add_node(start_node, end_node);

            if ((*code)[0] != RE_OP_END) {
                set_error(RE_ERROR_ILLEGAL, NULL);
                return FALSE;
            }
            ++(*code);

            *end = end_node;
            break;
        }
        case RE_OP_LOOKAROUND:
        {
            /* A lookaround. */
            RE_CODE positive;
            RE_PatternNode* lookahead_node;
            RE_PatternNode* start_sub;
            RE_PatternNode* end_sub;
            RE_PatternNode* success_node;

            positive = (*code)[1];
            if (positive != 0 && positive != 1) {
                set_error(RE_ERROR_ILLEGAL, NULL);
                return FALSE;
            }

            /* Create a node for the lookahead. */
            lookahead_node = create_node(pattern, RE_OP_LOOKAROUND, 1);
            if (!lookahead_node)
                return FALSE;
            lookahead_node->values[0] = positive;
            *code += 2;

            /* Compile the sequence and check that we've reached the end of the
             * subpattern.
             */
            if (!build_sequence(code, end_code, &start_sub, &end_sub, pattern))
                return FALSE;

            if ((*code)[0] != RE_OP_END) {
                set_error(RE_ERROR_ILLEGAL, NULL);
                return FALSE;
            }
            ++(*code);

            /* Create the 'SUCCESS' node and append it to the subpattern. */
            success_node = create_node(pattern, RE_OP_SUCCESS, 0);
            if (!success_node)
                return FALSE;

            /* Insert the subpattern into the node. */
            lookahead_node->next_2 = start_sub;
            add_node(end_sub, success_node);

            /* Append the lookahead. */
            add_node(*end, lookahead_node);
            *end = lookahead_node;
            break;
        }
        case RE_OP_REF_GROUP:
        case RE_OP_REF_GROUP_IGNORE:
        case RE_OP_REF_GROUP_REV:
        case RE_OP_REF_GROUP_IGNORE_REV:
        {
            /* A reference to a group. */
            int group;
            RE_PatternNode* node;

            group = (*code)[1];
            node = create_node(pattern, op, 1);
            if (!node)
                return FALSE;
            node->values[0] = group;
            *code += 2;

            /* Record that we have a reference to a group. */
            if (!record_ref_group(pattern, group))
                return FALSE;

            /* Append the reference. */
            add_node(*end, node);
            *end = node;
            break;
        }
        case RE_OP_SET:
        case RE_OP_SET_IGNORE:
        case RE_OP_SET_REV:
        case RE_OP_SET_IGNORE_REV:
        {
            /* A character set.
             *
             * The opcode is followed by a Boolean codeword (0 or 1).
             */
            RE_CODE positive;
            RE_PatternNode* node;

            positive = (*code)[1];
            if (positive != 0 && positive != 1) {
                set_error(RE_ERROR_ILLEGAL, NULL);
                return FALSE;
            }

            node = create_node(pattern, op, 1);
            if (!node)
                return FALSE;
            node->values[0] = (*code)[1];
            *code += 2;

            /* Compile the character set. */
            do {
                RE_CODE op;

                op = (*code)[0];
                switch (op) {
                case RE_OP_CATEGORY:
                {
                    /* A character category.
                     *
                     * The opcode is followed by a Boolean codeword (0 or 1).
                     */
                    RE_CODE positive;
                    RE_CODE value;

                    positive = (*code)[1];
                    value = (*code)[2];
                    if (positive != 0 && positive != 1) {
                        set_error(RE_ERROR_ILLEGAL, NULL);
                        return FALSE;
                    }

                    if (!add_set_member(node, op, 2))
                        return FALSE;
                    node->values[node->value_count++] = positive;
                    node->values[node->value_count++] = value;
                    *code += 3;
                    break;
                }
                case RE_OP_CHARACTER:
                    /* A character literal. */
                    if (!add_set_member(node, op, 1))
                        return FALSE;
                    node->values[node->value_count++] = (*code)[1];
                    *code += 2;
                    break;
                case RE_OP_RANGE:
                {
                    /* A character range. */
                    RE_CODE min_value;
                    RE_CODE max_value;

                    min_value = (*code)[1];
                    max_value = (*code)[2];
                    if (min_value > max_value) {
                        set_error(RE_ERROR_ILLEGAL, NULL);
                        return FALSE;
                    }
                    if (!add_set_member(node, op, 2))
                        return FALSE;
                    node->values[node->value_count++] = min_value;
                    node->values[node->value_count++] = max_value;
                    *code += 3;
                    break;
                }
                default:
                    /* Illegal opcode for a character set. */
                    set_error(RE_ERROR_ILLEGAL, NULL);
                    return FALSE;
                }
            } while (*code < end_code && (*code)[0] != RE_OP_END);

            /* Add the final 'END'. */
            if (!add_set_member(node, RE_OP_END, 0))
                return FALSE;

            /* Check that that we've reached the end correctly. (The last opcode
             * should be 'END'.)
             */
            if (*code >= end_code || (*code)[0] != RE_OP_END) {
                set_error(RE_ERROR_ILLEGAL, NULL);
                return FALSE;
            }
            ++(*code);

            /* Append the character set. */
            add_node(*end, node);
            *end = node;
            break;
        }
        default:
            /* We've found an opcode which we don't recognise. We'll leave it
             * for the caller.
             */
            if (*code >= end_code) {
                set_error(RE_ERROR_ILLEGAL, NULL);
                return FALSE;
            }

            return TRUE;
        }
    }

    /* If we get here then we're past the end of the regular expession code, but
     * the code should end with 'SUCCESS' (which isn't recognised by this
     * function), so we have an error.
     */
    set_error(RE_ERROR_ILLEGAL, NULL);
    return FALSE;
}

/* Compiles the regular expression code to 'nodes'.
 *
 * Various details about the regular expression are discovered during
 * compilation and stored in the PatternObject.
 */
Py_LOCAL(BOOL) compile_to_nodes(RE_CODE* code, RE_CODE* end_code, PatternObject*
  pattern) {
    RE_PatternNode* start;
    RE_PatternNode* end;
    RE_PatternNode* success_node;

    /* Compile a regex sequence and then check that we've reached the end
     * correctly. (The last opcode should be 'SUCCESS'.)
     *
     * If successful, 'start' and 'end' will point to the start and end nodes of
     * the compiled sequence.
     */
    if (!build_sequence(&code, end_code, &start, &end, pattern)) {
        destroy_nodes(pattern);
        return FALSE;
    }

    if (code + 1 != end_code || code[0] != RE_OP_SUCCESS) {
        set_error(RE_ERROR_ILLEGAL, NULL);
        destroy_nodes(pattern);
        return FALSE;
    }

    /* Create the 'SUCCESS' node and append it to the sequence. */
    success_node = create_node(pattern, RE_OP_SUCCESS, 0);
    if (!success_node) {
        destroy_nodes(pattern);
        return FALSE;
    }

    add_node(end, success_node);
    pattern->start_node = start;
    pattern->success_node = success_node;
    pattern->context_size = sizeof(RE_Context) - sizeof(RE_ContextValue) +
      (pattern->repeat_count + pattern->group_count) * sizeof(RE_ContextValue);

    /* Optimise the pattern. */
    if (!optimise_pattern(pattern)) {
        destroy_nodes(pattern);
        return FALSE;
    }

    return TRUE;
}

/* Compiles regular expression code to a PatternObject.
 *
 * The regular expression code is provided as a list and is then compiled to
 * 'nodes'. Various details about the regular expression are discovered during
 * compilation and stored in the PatternObject.
 */
static PyObject* re_compile(PyObject* self_, PyObject* args) {
    PyObject* pattern;
    int flags = 0;
    PyObject* code_list;
    PyObject* groupindex;
    PyObject* indexgroup;
    Py_ssize_t code_len;
    RE_CODE* code;
    Py_ssize_t i;
    PatternObject* self;

    if (!PyArg_ParseTuple(args, "OiOOO", &pattern, &flags, &code_list,
      &groupindex, &indexgroup))
        return NULL;

    /* Read the regular expression code. */
    code_len = PyList_GET_SIZE(code_list);
    code = PyMem_MALLOC(code_len * sizeof(*code));
    if (!code) {
        set_error(RE_ERROR_MEMORY, NULL);
        return NULL;
    }

    for (i = 0; i < code_len; i++) {
        PyObject* o = PyList_GET_ITEM(code_list, i);
        unsigned long value;
        if (PyInt_Check(o))
            value = PyInt_AsLong(o);
        else if (PyLong_Check(o))
            value = PyLong_AsUnsignedLong(o);
        else {
            set_error(RE_ERROR_ILLEGAL, NULL);
            break;
        }
        code[i] = value;
        if (code[i] != value) {
            set_error(RE_ERROR_ILLEGAL, NULL);
            break;
        }
    }

    if (PyErr_Occurred()) {
        PyMem_FREE(code);
        return NULL;
    }

    /* Create the PatternObject. */
    self = PyObject_NEW(PatternObject, &Pattern_Type);
    if (!self) {
        PyMem_FREE(code);
        return NULL;
    }

    /* Initialise the PatternObject. */
    self->pattern = pattern;
    self->flags = flags;
    self->weakreflist = NULL;
    self->start_node = NULL;
    self->success_node = NULL;
    self->repeat_count = 0;
    self->group_count = 0;
    self->ref_group_count = 0;
    self->groupindex = groupindex;
    self->indexgroup = indexgroup;
    self->node_capacity = 0;
    self->node_count = 0;
    self->node_list = NULL;
    self->group_info_capacity = 0;
    self->group_info_list = NULL;
    self->context_size = 0;
    self->compare_size = 0;
    Py_INCREF(self->pattern);
    Py_INCREF(self->groupindex);
    Py_INCREF(self->indexgroup);

    /* Compile the regular expression code to nodes. */
    if (!compile_to_nodes(code, code + code_len, self)) {
        Py_DECREF(self->pattern);
        Py_DECREF(self);
        PyMem_FREE(code);
        return NULL;
    }

    /* We no longer need the regular expression code. */
    PyMem_FREE(code);

    return (PyObject*)self;
}

/* Gets the size of the codewords. */
static PyObject* re_get_code_size(PyObject* self, PyObject* unused) {
    return Py_BuildValue("l", sizeof(RE_CODE));
}

/* Sets the exception to return on error. */
static PyObject* re_set_exception(PyObject* self_, PyObject* args) {
    if (!PyArg_ParseTuple(args, "O", &error_exception))
        return NULL;

    Py_INCREF(error_exception);

    Py_INCREF(Py_None);
    return Py_None;
}

/* The table of the module's functions. */
static PyMethodDef _functions[] = {
    {"compile", re_compile, METH_VARARGS},
    {"get_code_size", re_get_code_size, METH_NOARGS},
    {"set_exception", re_set_exception, METH_VARARGS},
    {NULL, NULL}
};

/* Initialises the module. */
PyMODINIT_FUNC init_regex(void) {
    PyObject* m;
    PyObject* d;
    PyObject* x;
#if defined(VERBOSE)
    /* Unbuffered in case it crashes! */
    setvbuf(stdout, NULL, _IONBF, 0);
#endif

    /* Patch object types. */
    Match_Type.ob_type = Scanner_Type.ob_type = Pattern_Type.ob_type =
      &PyType_Type;

    error_exception = PyExc_RuntimeError;

    m = Py_InitModule("_" RE_MODULE, _functions);
    if (!m)
        return;
    d = PyModule_GetDict(m);

    x = PyInt_FromLong(RE_MAGIC);
    if (x) {
        PyDict_SetItemString(d, "MAGIC", x);
        Py_DECREF(x);
    }

    x = PyInt_FromLong(sizeof(RE_CODE));
    if (x) {
        PyDict_SetItemString(d, "CODE_SIZE", x);
        Py_DECREF(x);
    }

    x = PyString_FromString(copyright);
    if (x) {
        PyDict_SetItemString(d, "copyright", x);
        Py_DECREF(x);
    }
}
#endif

/* vim:ts=4:sw=4:et */
